@@ -21,6 +21,7 @@
 #include <X11/cursorfont.h>
 #include <X11/ShellP.h>
 #include <X11/Xatom.h>
+#include <X11/Xft/Xft.h>
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -57,8 +58,6 @@ static XtResource resources[] =
      offset(drawHeight), XtRImmediate, (XtPointer) 512},
     {XtNlineWidth, XtCLineWidth, XtRInt, sizeof(int),
      offset(lineWidth), XtRImmediate, (XtPointer) 0},
-    {XtNfont, XtCFont, XtRFontStruct, sizeof(XFontStruct *),
-     offset(fontInfo), XtRString, XtDefaultFont},
     {XtNpattern, XtCPattern, XtRPixmap, sizeof(Pixmap),
      offset(pattern), XtRPixmap, (XtPointer) None},
     {XtNlinePattern, XtCLinePattern, XtRPixmap, sizeof(Pixmap),
@@ -444,8 +443,6 @@ RealizeProc(Widget w, Mask * valueMask,
 			   CapRound, Global.join);
 	if (pw->paint.pattern != None)
 	    XSetTile(XtDisplay(w), pw->paint.fgc, pw->paint.pattern);
-	if (pw->paint.fontInfo != NULL && pw->paint.fontInfo->fid != None)
-	    XSetFont(XtDisplay(w), pw->paint.fgc, pw->paint.fontInfo->fid);
 
 	/*
 	**  Then the line GC
@@ -1112,10 +1109,7 @@ SetValuesProc(Widget curArg, Widget request,
 			   new->paint.lineWidth, LineSolid,
 			   CapRound, Global.join);
     }
-    if (cur->paint.fontInfo->fid != new->paint.fontInfo->fid) {
-	XSetFont(XtDisplay(new), new->paint.fgc,
-		 new->paint.fontInfo->fid);
-    }
+
     if (cur->paint.drawWidth != new->paint.drawWidth ||
 	cur->paint.drawHeight != new->paint.drawHeight) {
 	UndoStack *cur;
@@ -1558,33 +1552,6 @@ PwUpdate(Widget w, XRectangle * rect, Boolean force)
     XFlush(XtDisplay(pw));
 }
 
-/*
-**  Update the current visual buffer with the contents
-**   of the undo area, thus a pseudo-real time undo.
-**   (it's not undo at all)
-**  Called from fontOp.c.
- */
-void
-PwUpdateFromLast(Widget w, XRectangle * rect)
-{
-    Pixmap pix;
-    PaintWidget pw = (PaintWidget) w;
-    Pixmap draw = GET_PIXMAP(pw);
-
-    if (pw->paint.paint != None)
-	pw = (PaintWidget) pw->paint.paint;
-
-    pix = pw->paint.undo->alphapix.pixmap;
-
-    XCopyArea(XtDisplay(w), pix, draw, pw->paint.gc,
-	      rect->x, rect->y, rect->width, rect->height,
-	      rect->x, rect->y);
-
-    memcpy(pw->paint.current.alpha, pw->paint.undo->alphapix.alpha,
-           rect->width * rect->height);
-    PwUpdate(w, rect, True);
-}
-
 /*
 **  Bogus XXX
  */
