@@ -14,10 +14,13 @@
 
 /* $Id: PaintEvent.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
+#include <X11/StringDefs.h>
 #include <X11/IntrinsicP.h>
 #include "PaintP.h"
 #include "xpaint.h"
 
+extern void PopdownMenusGlobal();
+
 typedef void (*func_t) (Widget, void *, XEvent *, OpInfo *);
 
 typedef struct data_s {
@@ -55,11 +58,18 @@ widgetEvent(Widget w, data_t * look)
 }
 #endif
 
+#define FROM_BELOW 1
+#if FROM_BELOW
+#define APPROX(u,m) (((u)/(m))*(m))
+#else
+#define APPROX(u,m) ((((u)+(m)/2)/(m))*(m))
+#endif
+
 static void 
 opHandleEvent(Widget w, XtPointer dataArg, XEvent * event, Boolean * junk)
 {
     data_t *data = (data_t *) dataArg;
-    int snap;
+    int snap_x, snap_y;
     PaintWidget paint = (PaintWidget) w;
     OpInfo *info = data->info;
     static Window lastWindow = None;
@@ -67,8 +77,29 @@ opHandleEvent(Widget w, XtPointer dataArg, XEvent * event, Boolean * junk)
     static int lastType;
     static int lastX, lastY;
     Boolean same;
+    Position x, y;
+    Dimension wd, ht;
     int zoom;
 
+    if (Global.popped_up) {
+        if (event->type == ButtonRelease) {
+	    XtVaGetValues(Global.popped_up,
+                          XtNx, &x, XtNy, &y, 
+                          XtNwidth, &wd, XtNheight, &ht, NULL);
+            x = event->xbutton.x_root - x;
+            y = event->xbutton.y_root - y;
+            if (x<0 || x>wd || y<0 || y>ht)
+                PopdownMenusGlobal();
+        }
+        event->type = None;
+        return;
+    }
+
+    XtVaGetValues((Widget)paint, XtNlocked, &same, NULL);
+    if (same) return;
+    XtVaGetValues((Widget)paint, XtNeditable, &same, NULL);
+    if (!same) return;
+
 #if 0
     XEvent myEvent;
 
@@ -80,14 +111,27 @@ opHandleEvent(Widget w, XtPointer dataArg, XEvent * event, Boolean * junk)
     **  Figure out if there is a snap, either by
     **   choice or by zooming
      */
-    if ((info->zoom = GET_ZOOM(paint)) == 0)
-	info->zoom = 1;
-    if ((info->isFat = ((zoom = GET_ZOOM(paint)) > 1)))
-	snap = GET_ZOOM(paint);
-    else if (paint->paint.snapOn)
-	snap = paint->paint.snap;
-    else
-	snap = 1;
+
+    if ((zoom = GET_ZOOM(paint)) == 0)
+	zoom = 1;
+    info->zoom = zoom;
+    info->isFat = ((zoom = GET_ZOOM(paint)) != 1);
+    if (paint->paint.snapOn) {
+        if (zoom>0) {
+	    snap_x = paint->paint.snap_x * zoom;
+	    snap_y = paint->paint.snap_y * zoom;
+	} else {
+	    snap_x = paint->paint.snap_x / (-zoom);
+	    snap_y = paint->paint.snap_y / (-zoom);
+	}
+    } else {
+        if (zoom>0)
+            snap_x = snap_y = zoom;
+        else
+            snap_x = snap_y = 1;
+    }
+    if (snap_x == 0) snap_x = 1;
+    if (snap_y == 0) snap_y = 1;
 
     same = (event->xany.window == lastWindow) &&
 	(event->xany.display == lastDisplay) &&
@@ -108,48 +152,18 @@ opHandleEvent(Widget w, XtPointer dataArg, XEvent * event, Boolean * junk)
     }
     if (((event->type == ButtonPress) ||
 	 (event->type == ButtonRelease) ||
-	 (event->type == MotionNotify)) && snap > 1) {
+	 (event->type == MotionNotify)) && (snap_x > 1 || snap_y > 1)) {
 	switch (event->type) {
 	case ButtonPress:
 	case ButtonRelease:
-#if 1
-	    event->xbutton.x = event->xbutton.x -
-		(event->xbutton.x % snap);
-	    event->xbutton.y = event->xbutton.y -
-		(event->xbutton.y % snap);
-#else
-	    t = event->xbutton.x;
-	    t = t / snap * snap;
-	    if (event->xbutton.x - t > snap / 2)
-		t += snap;
-	    event->xbutton.x = t;
-	    t = event->xbutton.y;
-	    t = t / snap * snap;
-	    if (event->xbutton.y - t > snap / 2)
-		t += snap;
-	    event->xbutton.y = t;
-#endif
+	    event->xbutton.x = APPROX(event->xbutton.x, snap_x);
+	    event->xbutton.y = APPROX(event->xbutton.y, snap_y);
 	    if (same && lastX == event->xbutton.x && lastY == event->xbutton.y)
 		return;
 	    break;
 	case MotionNotify:
-#if 1
-	    event->xmotion.x = event->xmotion.x -
-		(event->xmotion.x % snap);
-	    event->xmotion.y = event->xmotion.y -
-		(event->xmotion.y % snap);
-#else
-	    t = event->xbutton.x;
-	    t = t / snap * snap;
-	    if (event->xbutton.x - t > snap / 2)
-		t += snap;
-	    event->xbutton.x = t;
-	    t = event->xbutton.y;
-	    t = t / snap * snap;
-	    if (event->xbutton.y - t > snap / 2)
-		t += snap;
-	    event->xbutton.y = t;
-#endif
+	    event->xmotion.x = APPROX(event->xbutton.x, snap_x);
+	    event->xmotion.y = APPROX(event->xbutton.y, snap_y);
 	    if (same && lastX == event->xmotion.x && lastY == event->xmotion.y)
 		return;
 	    break;
@@ -161,11 +175,21 @@ opHandleEvent(Widget w, XtPointer dataArg, XEvent * event, Boolean * junk)
     }
 
     if (event->type == MotionNotify) {
-	info->x = (event->xmotion.x / zoom) + paint->paint.zoomX;
-	info->y = (event->xmotion.y / zoom) + paint->paint.zoomY;
+        if (zoom>0) {
+	    info->x = (event->xmotion.x / zoom) + paint->paint.zoomX;
+	    info->y = (event->xmotion.y / zoom) + paint->paint.zoomY;
+	} else {
+	    info->x = (event->xmotion.x * (-zoom) + (-zoom/2)) + paint->paint.zoomX;
+	    info->y = (event->xmotion.y * (-zoom) + (-zoom/2)) + paint->paint.zoomY;
+	}
     } else if (event->type == ButtonPress || event->type == ButtonRelease) {
-	info->x = (event->xbutton.x / zoom) + paint->paint.zoomX;
-	info->y = (event->xbutton.y / zoom) + paint->paint.zoomY;
+        if (zoom>0) {
+	    info->x = (event->xbutton.x / zoom) + paint->paint.zoomX;
+	    info->y = (event->xbutton.y / zoom) + paint->paint.zoomY;
+	} else {
+	    info->x = (event->xbutton.x * (-zoom) + (-zoom/2)) + paint->paint.zoomX;
+	    info->y = (event->xbutton.y * (-zoom) + (-zoom/2)) + paint->paint.zoomY;
+	}
     }
     /*
     **  In fatbits we can't tell the difference 
@@ -194,9 +218,10 @@ opHandleEvent(Widget w, XtPointer dataArg, XEvent * event, Boolean * junk)
     }
 
     if (data->surfMask & opWindow) {
+#if FROM_BELOW
 	int z = GET_ZOOM(paint) / 2;
 
-	if (z != 0) {
+	if (z > 0) {
 	    if (event->type == MotionNotify) {
 		event->xmotion.x += z;
 		event->xmotion.y += z;
@@ -205,6 +230,7 @@ opHandleEvent(Widget w, XtPointer dataArg, XEvent * event, Boolean * junk)
 		event->xbutton.y += z;
 	    }
 	}
+#endif
 	info->surface = opWindow;
 	info->drawable = event->xany.window;
 	data->func(w, data->data, event, info);
