@@ -1934,8 +1934,12 @@ PwRegionSet(Widget w, XRectangle * rect, Pixmap pix, Pixmap mask)
     **     rect == NULL, then this is just a "write" & "unmap" request
      */
     PwRegionFinish(w, True);
+    if (pw->paint.region.sourceImg) {
+        XDestroyImage(pw->paint.region.sourceImg);
+        pw->paint.region.sourceImg = NULL;
+    }
     if (rect == NULL)
-	return;
+        return;
 
     pw->paint.region.curZoom = zoom;
     XtVaSetValues(w, XtNtransparent, 0, NULL);
