@@ -87,8 +87,12 @@ press(Widget w, LocalInfo * l,
     l->ry = info->y;
     l->endX = l->startX = event->x;
     l->endY = l->startY = event->y;
-    SetCapAndJoin(w, info->first_gc, JoinMiter);
-    SetCapAndJoin(w, info->second_gc, JoinMiter);
+    SetCapAndJoin(w, info->first_gc,
+                  ((Global.cap)?Global.cap-1:CapButt),
+		  ((Global.join)?Global.join-1:JoinMiter));
+    SetCapAndJoin(w, info->second_gc,
+                  ((Global.cap)?Global.cap-1:CapButt),
+		  ((Global.join)?Global.join-1:JoinMiter));
 
     l->type = 2 * (boxStyle & 1) + ((event->state & ShiftMask) ? 1 : 0);
     l->drawn = 0;
