@@ -875,6 +875,9 @@ setStandardCursor(Widget w)
         XtVaSetValues(w, XtNcompress, False,
                       XtVaTypedArg, XtNcursor, XtRString, "spraycan", 
                       sizeof(Cursor), NULL);
+    else
+    if (CurrentOp->add == FontAdd)
+        SetIBeamCursor(w);
     else
 	SetCrossHairCursor(w);
 }
