@@ -79,6 +79,7 @@ typedef struct LocalInfo_s {
     Pixmap pixmap;
     Widget picker;
     Widget select;
+    Widget asbg;
     int dx, dy;
     Pixel bg;
 
@@ -310,6 +311,13 @@ motion(Widget w, LocalInfo * l, XEvent * event, Boolean * flg)
     int cx, cy;
     int dx, dy, dx2, dy2;
 
+    if (Global.popped_up) {
+        if (event->type == ButtonPress)
+	    PopdownMenusGlobal();
+        event->type = None;
+        return;
+    }
+
     while (XCheckTypedWindowEvent(XtDisplay(w), XtWindow(w),
 				  MotionNotify, (XEvent *) event));
 
@@ -366,7 +374,7 @@ scrollCB(Widget w, LocalInfo * l, XtPointer position)
 	l->isRGB = False;
     }
     if (!percent) return;
-    if ((int) position>0) 
+    if ((long)position>0) 
         *percent += 1.0/256.0;
     else
         *percent -= 1.0/256.0;
@@ -724,7 +732,11 @@ colorPicker(Widget parent, Colormap cmap, Pixel * pixval)
 				     XtNfromVert, button,
 				     XtNfromHoriz, picker,
 				     NULL);
-    XtAddCallback(button, XtNcallback, (XtCallbackProc) grabCB, (XtPointer) l);
+    l->asbg = XtVaCreateWidget("asbg", commandWidgetClass, form,
+				     XtNfromVert, l->select,
+				     XtNfromHoriz, picker,
+				     NULL);
+    XtAddCallback(button, XtNcallback, (XtCallbackProc)grabCB, (XtPointer) l);
 
     /*
     **	Now create the scroll bars
@@ -901,6 +913,19 @@ ColorPickerGetSelectWidget(Widget w)
     return l->select;
 }
 
+Widget
+ColorPickerGetAsBgWidget(Widget w)
+{
+    LocalInfo *l = head;
+
+    for (; l != NULL && l->form != w; l = l->next);
+
+    if (l == NULL) return NULL;
+
+    XtManageChild(l->asbg);
+    return l->asbg;
+}
+
 Pixel
 ColorPickerGetPixel(Widget w)
 {
