@@ -86,6 +86,13 @@ AlertBox(Widget parent, char *msg, XtCallbackProc okProc,
     Arg args[6];
     
     XtVaGetValues(GetShell(parent), XtNx, &x, XtNy, &y, NULL);
+    x += 24; y += 24;
+    if (x<24) x=24;
+    if (x>WidthOfScreen(XtScreen(parent))-260)
+       x=WidthOfScreen(XtScreen(parent))-260;
+    if (y<24) y=24;
+    if (y>HeightOfScreen(XtScreen(parent))-84)
+       y=HeightOfScreen(XtScreen(parent))-84;
 
     XtSetArg(args[nargs], XtNx, x); nargs++;
     XtSetArg(args[nargs], XtNy, y); nargs++;
