@@ -165,7 +165,7 @@ press(Widget w, LocalInfo *l, XButtonEvent *event, OpInfo *info)
         save = Global.dashnumber;
         Global.dashnumber = 0;
         SetCapAndJoin(w, l->gc,
-                ((Global.cap)?Global.cap-1:CapButt),
+                ((Global.cap)?Global.cap-1:CapRound),
 		((Global.join)?Global.join-1:JoinRound));
         Global.dashnumber = save;
 
