@@ -131,7 +131,7 @@ CreateFatCursor(Widget paint, Pixmap cursor, FatLineCursor * fc)
 
     fc->nsegs = 0;
     fc->size = FATCHUNK;
-    fc->segs = malloc(FATCHUNK * sizeof(XSegment));
+    fc->segs = xmalloc(FATCHUNK * sizeof(XSegment));
 
     GetPixmapWHD(dpy, cursor, &w, &h, NULL);
     src = XGetImage(dpy, cursor, 0, 0, w, h, AllPlanes, ZPixmap);
@@ -186,16 +186,22 @@ FatCursorDraw(Widget w, FatCursorInfo * l, XMotionEvent * event)
 	x = l->lastX;
 	y = l->lastY;
 	s = master.cursor.segs;
-	for (i = 0; i < n; ++i, ++s)
-	    XDrawLine(dpy, win, l->gcx,
-		      x + z * s->x1, y + z * s->y1,
-		      x + z * s->x2, y + z * s->y2);
+        if (z>0)
+	    for (i = 0; i < n; ++i, ++s)
+	        XDrawLine(dpy, win, l->gcx,
+		          x + z * s->x1, y + z * s->y1,
+		          x + z * s->x2, y + z * s->y2);
+        else
+	    for (i = 0; i < n; ++i, ++s)
+	        XDrawLine(dpy, win, l->gcx,
+			  x + s->x1/(-z), y + s->y1/(-z),
+			  x + s->x2/(-z), y + s->y2/(-z));
     }
     /*
      * If a button is pressed, leave the cursor undrawn
      */
     if (event->type == ButtonPress) {
-	drawing = 1;
+    	drawing = 1;
 	return;
     }
     /*
@@ -209,20 +215,33 @@ FatCursorDraw(Widget w, FatCursorInfo * l, XMotionEvent * event)
     l->drawn = 1;
     x = event->x;
     y = event->y;
-    /* Snap to zoom grid */
-    x = x / z * z;
-    y = y / z * z;
-#if 0
-    if (event->x - x > z / 2)
-	x += z;
-    if (event->y - y > z / 2)
-	y += z;
-#endif
-    s = master.cursor.segs;
-    for (i = 0; i < n; ++i, ++s)
-	XDrawLine(dpy, win, l->gcx,
-		  x + z * s->x1, y + z * s->y1,
-		  x + z * s->x2, y + z * s->y2);
+
+    if (z>0) {
+        /* Snap to zoom grid */
+        x = x / z * z;
+        y = y / z * z; 
+	/* This can be confusing, so just forget it ... */
+        /*
+        if (event->x - x > z / 2)
+	    x += z;
+        if (event->y - y > z / 2)
+	    y += z;
+	*/
+        s = master.cursor.segs;
+        for (i = 0; i < n; ++i, ++s)
+	    XDrawLine(dpy, win, l->gcx,
+		      x + z * s->x1, y + z * s->y1,
+		      x + z * s->x2, y + z * s->y2);
+    } else {
+        /* Snap to zoom grid */
+        x = x * z * z;
+        y = y * z * z;
+        s = master.cursor.segs;
+        for (i = 0; i < n; ++i, ++s)
+	    XDrawLine(dpy, win, l->gcx,
+		      x + s->x1/(-z), y + s->y1/(-z),
+		      x + s->x2/(-z), y + s->y2/(-z));
+    }
 
     l->lastX = x;
     l->lastY = y;
@@ -260,7 +279,8 @@ FatCursorAddZoom(int zoom, Widget winwid)
      * it can remove the cursor in time.
      */
     XtInsertEventHandler(winwid,
-			 PointerMotionMask | ButtonPressMask | ButtonReleaseMask | LeaveWindowMask,
+		  PointerMotionMask | LeaveWindowMask |
+		  ButtonPressMask | ButtonReleaseMask, 
 		  False, (XtEventHandler) FatCursorDraw, (XtPointer) new,
 			 XtListHead);
 }
@@ -393,10 +413,17 @@ moveCursor(LocalInfo * l, int x, int y, int w, int h)
     XtVaGetValues(l->paint, XtNdrawWidth, &dw, XtNdrawHeight, &dh, NULL);
 
     z = l->zoom;
-    h *= z;
-    w *= z;
-    dh *= z;
-    dw *= z;
+    if (z>0) {
+        h *= z;
+        w *= z;
+        dh *= z;
+        dw *= z;
+    } else {
+        h /= -z;
+        w /= -z;
+        dh /= -z;
+        dw /= -z;
+    }
     if (x < 0)
 	x = 0;
     if (y < 0)
@@ -408,10 +435,17 @@ moveCursor(LocalInfo * l, int x, int y, int w, int h)
     if (x < 0 || y < 0)
 	return;
 
-    rx = x / z;		/* correct for parent's zoom factor */
-    ry = y / z;
-    x = rx * z;		/* snap view origin to zoom grid */
-    y = ry * z;
+    if (z>0) {
+        rx = x / z;		/* correct for parent's zoom factor */
+        ry = y / z;
+        x = rx * z;		/* snap view origin to zoom grid */
+        y = ry * z;
+    } else {
+        rx = x * (-z);		/* correct for parent's zoom factor */
+        ry = y * (-z);
+        x = rx / (-z);		/* snap view origin to zoom grid */
+        y = ry / (-z);
+    }
 
     XtVaSetValues(l->view, XtNzoomX, rx, XtNzoomY, ry, NULL);
     XtVaSetValues(l->cursor, XtNx, x - l->spX - l->spBW - 1,
@@ -526,8 +560,14 @@ resizeCursor(LocalInfo * l)
 	}
 	moveCursor(l, nx, ny, pw, ph);
     }
-    pw *= l->zoom;
-    ph *= l->zoom;
+
+    if (l->zoom>0) {
+        pw *= l->zoom;
+        ph *= l->zoom;
+    } else {
+        pw /= -l->zoom;
+        ph /= -l->zoom;
+    }
 
     XtVaSetValues(l->cursor, XtNwidth, pw + 2 * (PADDING + BW),
 		  XtNheight, ph + 2 * (PADDING + BW),
@@ -542,8 +582,12 @@ static void
 press(Widget w, LocalInfo * l, XButtonEvent * event, Boolean * flg)
 {
     Position x, y;
+    static long prev_time = 0;
 
     XtVaGetValues(w, XtNx, &x, XtNy, &y, NULL);
+    if (abs(event->time-prev_time) < 300)
+        XMapRaised(XtDisplay(l->shell), XtWindow(l->shell));
+    prev_time = event->time;
 
     l->offX = event->x;
     l->offY = event->y;
@@ -677,7 +721,7 @@ FatbitsUpdate(Widget w, int zoom)
     /* find fatbits info structure for this popup */
     for (l = head; l != NULL; l = l->next)
 	if (l->paint == w) {
-	    if (zoom > 0)
+	    if (zoom != 0)
 		l->zoom = zoom;	/* new parent zoom factor */
 	    else
 		zoom = l->zoom;
@@ -686,15 +730,21 @@ FatbitsUpdate(Widget w, int zoom)
 
 	    /* reposition cursor window in parent paint window */
 	    XtVaGetValues(l->view, XtNzoomX, &x, XtNzoomY, &y, NULL);
-	    XtVaSetValues(l->cursor, XtNx, x * zoom - l->spX - l->spBW - 1,
-			  XtNy, y * zoom - l->spY - l->spBW - 1, NULL);
+            if (zoom>0)
+	        XtVaSetValues(l->cursor, XtNx, x * zoom - l->spX - l->spBW - 1,
+			      XtNy, y * zoom - l->spY - l->spBW - 1, NULL);
+            else
+	        XtVaSetValues(l->cursor,
+                              XtNx, x / (-zoom) - l->spX - l->spBW - 1,
+			      XtNy, y / (-zoom) - l->spY - l->spBW - 1, NULL);
 
 	    XtVaGetValues(w, XtNdrawWidth, &dw, XtNdrawHeight, &dh, NULL);
 	    /* The Viewport is grandparent of the paint widget */
 	    vw = vh = 0;
 	    XtVaGetValues(XtParent(XtParent(w)),
 			  XtNwidth, &vw, XtNheight, &vh, NULL);
-	    if ((dw * zoom > vw) || (dh * zoom > vh))
+            if (zoom>0) {
+	        if ((dw * zoom > vw) || (dh * zoom > vh))
 		/*
 		 * The zoomed canvas is larger than the viewport.
 		 * Reposition view on canvas so cursor window remains visible.
@@ -702,12 +752,47 @@ FatbitsUpdate(Widget w, int zoom)
 		 * positioned at (0,0) in the viewport window.
 		 * Still, it's better than having a non-visible cursor window.
 		 */
-		XawViewportSetCoordinates(XtParent(XtParent(w)), x * zoom, y * zoom);
+		 XawViewportSetCoordinates(XtParent(XtParent(w)), 
+                                              x * zoom, y * zoom);
 
-	    return;		/* there can only be one fatbits popup */
+	         return;	/* there can only be one fatbits popup */
+	    }
+
+            if (zoom<0) {
+	        if ((dw /(-zoom) > vw) || (dh / (-zoom) > vh))
+		    XawViewportSetCoordinates(XtParent(XtParent(w)), 
+                                              x / (-zoom), y / (-zoom));
+
+	        return;
+	    }
 	}
 }
 
+void 
+keyPress(Widget w, void * l, XKeyEvent * event, void * info)
+{
+    KeySym keysym;
+    int len;
+    char buf[4];
+
+    len = XLookupString(event, buf, sizeof(buf) - 1, &keysym, NULL);
+
+    switch (keysym) {
+        case XK_Escape:
+            PopdownMenusGlobal();
+            break;
+        default:
+	    break;
+    }
+}
+
+void FatbitsEditDestroy(Widget paint)
+{
+    LocalInfo *l;
+    for (l = head; l != NULL && l->paint != paint; l = l->next);
+    if (l != NULL) 
+        doneCallback(l->shell, l, NULL);
+}
 
 /*
 **  Construct the fatbits popup.
@@ -721,6 +806,8 @@ FatbitsEdit(Widget paint)
     Colormap cmap;
     Position x, y, lx, ly;
     Dimension width = 48, height = 48;
+    WidgetList wlist;
+
     LocalInfo *l;
     Arg args[4];
     int nargs = 0;
@@ -775,7 +862,7 @@ FatbitsEdit(Widget paint)
 
     XtAddCallback(button, XtNcallback, doneCallback, (XtPointer) l);
 
-    zoombut = (Widget *) malloc(XtNumber(zoomList) * sizeof(Widget));
+    zoombut = (Widget *) xmalloc(XtNumber(zoomList) * sizeof(Widget));
     for (i = 0; i < XtNumber(zoomList); i++) {
 	zoombut[i] = XtVaCreateManagedWidget(zoomList[i],
 					 toggleWidgetClass, form,
@@ -817,7 +904,6 @@ FatbitsEdit(Widget paint)
 
     XtVaSetValues(fat, XtNwidth, width * zoom, XtNheight, height * zoom, NULL);
 
-
     XtAddCallback(shell, XtNdestroyCallback, destroyCallback, (XtPointer) l);
     AddDestroyCallback(shell, (DestroyCallbackFunc) doneCallback, (XtPointer) l);
 
@@ -878,17 +964,26 @@ FatbitsEdit(Widget paint)
     /*
     **	Set the current operator
      */
-    GraphicAdd(fat);
     StateAddParent(shell, paint);
 
     XtUnmanageChild(button);
     XMapWindow(XtDisplay(button), XtWindow(button));
     XtUnmanageChild(box);
     XMapWindow(XtDisplay(box), XtWindow(box));
-
+    XFlush(XtDisplay(box));
+    
     for (i=0; i<XtNumber(zoomList); i++) {
-      XtUnmanageChild(zoombut[i]);
-      XMapWindow(XtDisplay(zoombut[i]), XtWindow(zoombut[i]));
+        XtUnmanageChild(zoombut[i]);
+        XMapWindow(XtDisplay(zoombut[i]), XtWindow(zoombut[i]));
     }
+    XFlush(XtDisplay(box));
+    usleep(30000);
+    XtSetMinSizeHints(l->shell, 214, 108);
+    XtAddEventHandler(form, KeyPressMask | KeyReleaseMask,
+		      False, (XtEventHandler) keyPress, NULL);
+
+    wlist = InitializedWidgetList();
+    XtVaSetValues(fat, XtNmenuwidgets, wlist, NULL);
+    CheckMarkItems(fat, wlist);
     return;
 }
