@@ -298,9 +298,10 @@ typedef struct arg_s {
            form, title_w, name, vport, 
            home, root, dot, hidden, cwd_w, info, ok, cancel,
            refresh, delete, edit, create,
-           dpilabel, dpi, zoomlabel, zoom, alphalabel, alpha,
+           dpilabel, dpi, zoomlabel, zoom,
+           alphalabel, alphaleft, alpha, alpharight,
            pagelabel, page, pagetotal, pageleft, pageLeft, pageright, pageRight,
-           bbox;
+           bboxlabel, bboxleft, bbox, bboxright;
     Widget format[NUM_FORMATS+2];
     int first, numformat, oldwidth;
     /*
@@ -377,12 +378,13 @@ browserResized(Widget w, arg_t * l, XConfigureEvent * event, Boolean * flag)
     XtVaSetValues(l->parent, XtNwidth, width, XtNheight, height, NULL);   
     XtResizeWidget(l->name, width1-2, 20, 1);
     XtResizeWidget(l->form, width1 + 10, height, 0);
-    XtVaGetValues(l->list, XtNlongest, &i, NULL);
-    /* the next 4 lines avoid a crash with certain buggy Xaw3d versions */
-    XtVaGetValues(l->list, XtNwidth, &width2, NULL);
-    if (width2 > i+25) {
-        width2 = i+25;
-        XtVaSetValues(l->list, XtNwidth, width2, NULL);
+    if (Global.numfiles) {
+        XtVaGetValues(l->list, XtNlongest, &i, NULL);
+        XtVaGetValues(l->list, XtNwidth, &width2, NULL);
+        if (width2 > i+25) {
+            width2 = i+25;
+            XtVaSetValues(l->list, XtNwidth, width2, NULL);
+	}
     }
     XtResizeWidget(l->vport, width1, height1, 1);     
     XtVaSetValues(l->vport, XtNwidth, width1, XtNheight, height1, NULL);
@@ -408,13 +410,13 @@ browserResized(Widget w, arg_t * l, XConfigureEvent * event, Boolean * flag)
     XtMoveWidget(l->cwd_w, x, height-158);
     XtMoveWidget(l->info, x, height-134);
 
-    im = &l->bbox - &l->refresh;
+    im = &l->bboxright - &l->refresh;
     widget = &l->refresh;
     for (i=0; i<=im; i++) {
         XtVaGetValues(widget[i], XtNx, &x, NULL);
         if (i==0) j = 5-x;
         XtMoveWidget(widget[i], x+j, height - 
-                     ((i<=3)? 99: ((i<=9)?67:35)));
+                     ((i<=3)? 99: ((i<=11)?67:35)));
     }
 }
 
@@ -426,9 +428,9 @@ doDirname(arg_t * arg, char *file)
 
     if (file == NULL) {
 #ifdef NOSTDHDRS
-	getwd(newPath);
+        (void) getwd(newPath);
 #else
-	getcwd(newPath, sizeof(newPath));
+	(void) getcwd(newPath, sizeof(newPath));
 #endif
 	return newPath;
     }
@@ -542,7 +544,7 @@ setCWD(arg_t * arg, char *dir)
     if (arg->browserType == LOADED) {
         emptyList(arg);
         fileCount = Global.numfiles;
-        list = (String *) XtCalloc(sizeof(String *), fileCount+1);
+        list = (String *) XtCalloc(sizeof(String *), fileCount+2);
         for (i=0; i<fileCount; i++) {
             filePtr = Global.loadedfiles[i];
             if (*filePtr == '/') {
@@ -554,11 +556,13 @@ setCWD(arg_t * arg, char *dir)
 	        sprintf(list[i], "%s/%s", arg->dirname, Global.loadedfiles[i]);
 	    }
         }
+        if (fileCount == 0)
+	    list[0] = strdup("");
 	XtVaSetValues(arg->cwd_w, XtNlabel, msgText[LIST_LOADED_FILES], NULL);
         sprintf(infoStr, "%d %s", fileCount, msgText[FILES]);
         i = fileCount;
-        if (Global.numfiles==1) 
-	    XtVaSetValues(arg->name, XtNstring, Global.loadedfiles[0], NULL);
+        if (Global.numfiles<=1) 
+	    XtVaSetValues(arg->name, XtNstring, list[0], NULL);
         goto fill;
     }
 
@@ -644,6 +648,7 @@ setCWD(arg_t * arg, char *dir)
         dirCount, msgText[DIRECTORIES], fileCount, msgText[FILES]);
 
   fill:
+
     XawListChange(arg->list, list, i, 0, True);
     XtVaSetValues(arg->info, XtNlabel, infoStr, NULL);
 
@@ -722,16 +727,19 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
     arg->isPopped = False;
     arg->isToRefresh = False;
     arg->oldwidth = 0;
+
     if (arg->browserType == LOADED) {
-        char *dpistr, *pagestr, *zoomstr, *alphastr;
+      char *dpistr, *pagestr, *zoomstr, *alphastr, *bboxstr;
         Widget paint = GetNonDirtyCanvas();
         XtVaGetValues(arg->dpi, XtNstring, &dpistr, NULL);
         XtVaGetValues(arg->page, XtNstring, &pagestr, NULL);
         XtVaGetValues(arg->zoom, XtNstring, &zoomstr, NULL);
         XtVaGetValues(arg->alpha, XtNstring, &alphastr, NULL);
+        XtVaGetValues(arg->bbox, XtNstring, &bboxstr, NULL);
         Global.dpi = atof(dpistr);
         Global.numpage = atoi(pagestr);
         Global.alpha_mode = abs(atoi(alphastr))%4;
+        file_bbox = abs(atoi(bboxstr))%4;
         zoom = StrToZoom(zoomstr);
         if (zoom>32) zoom = 32;
         if (zoom<-9) zoom = -9;
@@ -785,6 +793,9 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
                           RWGetMsg());
 	    else
 		arg->okFunc(arg->w, (XtPointer) file, (XtPointer) image);
+            if (Global.numfiles)
+                setCWD(arg, NULL);
+            browserResized(arg->parent, arg, NULL, False);
 	}
         StateSetBusy(False);
         Global.default_zoom = zoom_prev;
@@ -797,7 +808,10 @@ alphaCallback(Widget w, XtPointer argArg, XtPointer junk)
     arg_t *arg = (arg_t *) argArg;
     static char value[4];
     if (!file_transparent) return;
-    Global.alpha_mode = (Global.alpha_mode+1)%4;
+    if (strstr(XtName(w), "xright"))
+        Global.alpha_mode = (Global.alpha_mode+1)%4;
+    else
+        Global.alpha_mode = (Global.alpha_mode+3)%4;
     sprintf(value, "%d",  Global.alpha_mode);
     XtVaSetValues(arg->alpha, XtNstring, value, NULL);
     okCallback(w, argArg, junk);
@@ -830,12 +844,18 @@ pageCallback(Widget w, XtPointer argArg, XtPointer junk)
 }
 
 static void
-bboxToggleCallback(Widget w, XtPointer wlArg, XtPointer junk)
+bboxCallback(Widget w, XtPointer argArg, XtPointer junk)
 {
-     Boolean state;
-     XtVaGetValues(w, XtNstate, &state, NULL);
-     file_bbox = (state)? 1:0;
-     okCallback(w, wlArg, junk);
+    arg_t *arg = (arg_t *) argArg;
+    char name[2];
+    if (strstr(XtName(w), "xright"))
+        file_bbox = (file_bbox+1)%4;
+    else
+        file_bbox = (file_bbox+3)%4;
+    name[0] = '0'+file_bbox;
+    name[1] = '\0';
+    XtVaSetValues(arg->bbox, XtNstring, name, NULL);
+    okCallback(w, argArg, junk);
 }
 
 static void 
@@ -874,9 +894,9 @@ dotCallback(Widget w, XtPointer argArg, XtPointer itemArg)
 {
     arg_t *arg = (arg_t *) argArg;
 #ifdef NOSTDHDRS
-    getwd(arg->dirname);
+    (void) getwd(arg->dirname);
 #else
-    getcwd(arg->dirname, MAX_PATH);
+    (void) getcwd(arg->dirname, MAX_PATH);
 #endif
     setCWD(arg, NULL);
     browserResized(arg->parent, arg, NULL, False); 
@@ -886,7 +906,8 @@ static void
 refreshCallback(Widget w, XtPointer argArg, XtPointer itemArg)
 {
     arg_t *arg = (arg_t *) argArg;
-    setCWD(arg, NULL);
+    if (Global.numfiles)
+        setCWD(arg, NULL);
     browserResized(arg->parent, arg, NULL, False); 
 }
 
@@ -895,11 +916,30 @@ fileDeleteOkCallback(Widget w, XtPointer argArg,  XtPointer infoArg)
 {
     arg_t *arg = (arg_t *) argArg;
     char *name;
+    int i, exists;
+    struct stat buf;
 
     XtVaGetValues(arg->name, XtNstring, &name, NULL);
-    unlink(name);
-    RemoveFileToGlobalList(name);
-    setCWD(arg, NULL);
+    if (!name || !*name) return;
+    if (stat(name, &buf)) 
+        Notice(w, msgText[FILE_DOES_NOT_EXIST], name);
+    else {
+        exists = 0;
+        for (i=0; i<Global.numfiles; i++)
+	    if (!strcmp(name, Global.loadedfiles[i])) {
+	        exists = 1;
+	        break;
+	    }
+        if (exists) {
+            unlink(name);
+            RemoveFileFromGlobalList(name);
+	} else {
+            Notice(w, msgText[FILE_NOT_IN_THE_LIST], name);
+  	    return;
+	}
+    }
+    if (Global.numfiles)
+        setCWD(arg, NULL);
     browserResized(arg->parent, arg, NULL, False); 
 }
 
@@ -912,10 +952,11 @@ static void
 fileDeleteCallback(Widget w, XtPointer argArg,  XtPointer junk2)
 {
     arg_t *arg = (arg_t *) argArg;
-    char *name;
+    char *name = NULL;
     char title[300];
 
     XtVaGetValues(arg->name, XtNstring, &name, NULL);
+    if (!name || !*name) return;
     sprintf(title, "%s : %s", msgText[REMOVE_FILE], name);
 
     AlertBox(w, title, fileDeleteOkCallback, exitDeleteCallback, arg);
@@ -970,7 +1011,7 @@ createLXPCallback(Widget w, XtPointer argArg, XtPointer fileArg)
 
     sprintf(buf, "cd %s ; tar cvfz %s .\n", dir, file);
     free(dir);
-    system(buf);
+    (void) system(buf);
     AddFileToGlobalList(file);
     setCWD(arg, NULL);
     browserResized(arg->parent, arg, NULL, False);
@@ -1046,9 +1087,9 @@ buildBrowser(Widget parent, char *title, arg_t * arg,
 	toglt = XtParseTranslationTable("<BtnDown>,<BtnUp>: set() notify()");
     }
 #ifdef NOSTDHDRS
-    getwd(arg->dirname);
+    (void) getwd(arg->dirname);
 #else
-    getcwd(arg->dirname, sizeof(arg->dirname));
+    (void) getcwd(arg->dirname, sizeof(arg->dirname));
 #endif
 
     form = XtVaCreateManagedWidget("browser",
@@ -1195,7 +1236,7 @@ buildBrowser(Widget parent, char *title, arg_t * arg,
 }
 
 static Widget
-buildOpenBrowser(Widget w, arg_t * arg, int width, int height)
+buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
 {
     Widget shell, browser, okButton, cancelButton, form;
     Widget toggle, firstToggle = None;
@@ -1210,7 +1251,10 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height)
     
     
     shell = XtVisCreatePopupShell("filebrowser",
-				  transientShellWidgetClass, GetToplevel(w),
+				  ((mode==LOADED)?
+				   topLevelShellWidgetClass:
+                                   transientShellWidgetClass),
+                                  GetToplevel(w),
 				  args, nargs);
     arg->shell = shell;
     form = XtVaCreateManagedWidget("form",
@@ -1348,7 +1392,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height)
 				       XtNvertDistance, 10,
                                        XtNfromHoriz, arg->dpilabel,
 				       XtNhorizDistance, 10,
-				       XtNwidth, 48,
+				       XtNwidth, 50,
 				       XtNheight, 20,
 				       NULL);
         sprintf(value, "%g", Global.dpi);
@@ -1373,14 +1417,14 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height)
 				       XtNvertDistance, 10,
                                        XtNfromHoriz, arg->zoomlabel,
 				       XtNhorizDistance, 10,
-				       XtNwidth, 48,
+				       XtNwidth, 50,
 				       XtNheight, 20,
 				       NULL);
         XtVaSetValues(arg->zoom, XtNstring, ZoomToStr(Global.default_zoom),
                       NULL);
 
         arg->alphalabel = XtVaCreateManagedWidget("alpha",
-				       commandWidgetClass, browser,
+				       labelWidgetClass, browser,
 				       XtNborderWidth, 0,
                                        XtNfromVert, arg->refresh,
 				       XtNvertDistance, 10,
@@ -1388,6 +1432,16 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height)
 				       XtNhorizDistance, 16,
 				       NULL);
 
+        arg->alphaleft = XtVaCreateManagedWidget("xxleft",
+				       commandWidgetClass, browser,
+                                       XtNfromVert, arg->refresh,
+				       XtNvertDistance, 7,
+				       XtNfromHoriz, arg->alphalabel,
+				       XtNhorizDistance, 3,
+				       XtNheight, 20,
+				       XtNwidth, 18,
+				       NULL);
+
         arg->alpha = XtVaCreateManagedWidget("alphaW",
 				       asciiTextWidgetClass, browser,
  				       XtNeditType, XawtextEdit,
@@ -1396,12 +1450,22 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height)
 				       XtNtranslations, trans,
                                        XtNfromVert, arg->refresh,
 				       XtNvertDistance, 10,
-                                       XtNfromHoriz, arg->alphalabel,
-				       XtNhorizDistance, 10,
-				       XtNwidth, 24,
+                                       XtNfromHoriz, arg->alphaleft,
+				       XtNhorizDistance, 3,
+				       XtNwidth, 18,
 				       XtNheight, 20,
 				       NULL);
 
+        arg->alpharight = XtVaCreateManagedWidget("xxright",
+				       commandWidgetClass, browser,
+                                       XtNfromVert, arg->refresh,
+				       XtNvertDistance, 7,
+				       XtNfromHoriz, arg->alpha,
+				       XtNhorizDistance, 3,
+				       XtNheight, 20,
+				       XtNwidth, 18,
+				       NULL);
+
         Global.alpha_mode = abs(Global.alpha_mode)%4;
         sprintf(value, "%d", Global.alpha_mode);
         XtVaSetValues(arg->alpha, XtNstring, value, NULL);
@@ -1414,34 +1478,35 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height)
 				       XtNhorizDistance, 10,
 				       NULL);
 
-        sprintf(value, "/ %d", file_numpages);
+        sprintf(value, "/%d", file_numpages);
         arg->pagetotal = XtVaCreateManagedWidget(value,
 				       labelWidgetClass, browser,
 				       XtNborderWidth, 0,
+				       XtNwidth, 31,
                                        XtNfromVert, arg->dpi,
 				       XtNvertDistance, 7,
 				       XtNfromHoriz, arg->pagelabel,
 				       XtNhorizDistance, 0,
 				       NULL);
 
-        arg->pageLeft = XtVaCreateManagedWidget("\253",
+        arg->pageLeft = XtVaCreateManagedWidget("xxleft",
 				       commandWidgetClass, browser,
                                        XtNfromVert, arg->dpi,
 				       XtNvertDistance, 7,
 				       XtNfromHoriz, arg->pagetotal,
 				       XtNhorizDistance, 10,
 				       XtNheight, 20,
-				       XtNwidth, 20,
+				       XtNwidth, 18,
 				       NULL);
 
-        arg->pageleft = XtVaCreateManagedWidget("<",
+        arg->pageleft = XtVaCreateManagedWidget("xleft",
 				       commandWidgetClass, browser,
                                        XtNfromVert, arg->dpi,
 				       XtNvertDistance, 7,
 				       XtNfromHoriz, arg->pageLeft,
 				       XtNhorizDistance, 3,
 				       XtNheight, 20,
-				       XtNwidth, 20,
+				       XtNwidth, 18,
 				       NULL);
 
         arg->page = XtVaCreateManagedWidget("pageW",
@@ -1453,50 +1518,87 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height)
                                        XtNfromVert, arg->dpi,
 				       XtNvertDistance, 7,
                                        XtNfromHoriz, arg->pageleft,
-				       XtNhorizDistance, 10,
+				       XtNhorizDistance, 3,
 				       XtNwidth, 48,
 				       NULL);
 
-        arg->pageright = XtVaCreateManagedWidget(">",
+        arg->pageright = XtVaCreateManagedWidget("xright",
 				       commandWidgetClass, browser,
                                        XtNfromVert, arg->dpi,
 				       XtNvertDistance, 7,
 				       XtNfromHoriz, arg->page,
-				       XtNhorizDistance, 10,
+				       XtNhorizDistance, 3,
 				       XtNheight, 20,
-				       XtNwidth, 20,
+				       XtNwidth, 18,
 				       NULL);
 
-        arg->pageRight = XtVaCreateManagedWidget("\273",
+        arg->pageRight = XtVaCreateManagedWidget("xxright",
 				       commandWidgetClass, browser,
                                        XtNfromVert, arg->dpi,
 				       XtNvertDistance, 7,
 				       XtNfromHoriz, arg->pageright,
 				       XtNhorizDistance, 3,
 				       XtNheight, 20,
-				       XtNwidth, 20,
+				       XtNwidth, 18,
 				       NULL);
 
-        arg->bbox = XtVaCreateManagedWidget("BBox",
-				       toggleWidgetClass, browser,
+        arg->bboxlabel = XtVaCreateManagedWidget("BBox",
+				       labelWidgetClass, browser,
 				       XtNborderWidth, 0,
-				       XtNstate, (file_bbox)?True:False,
                                        XtNheight, 20,
                                        XtNfromVert, arg->dpi,
 				       XtNvertDistance, 7,
                                        XtNfromHoriz, arg->pageRight,
 				       XtNhorizDistance, 10,
 				       NULL);
-        XtAddCallback(arg->bbox, XtNcallback, 
-                   (XtCallbackProc) bboxToggleCallback, (XtPointer) arg);
+
+        arg->bboxleft = XtVaCreateManagedWidget("xxleft",
+				       commandWidgetClass, browser,
+                                       XtNfromVert, arg->dpi,
+				       XtNvertDistance, 7,
+				       XtNfromHoriz, arg->bboxlabel,
+				       XtNhorizDistance, 3,
+				       XtNheight, 20,
+				       XtNwidth, 18,
+				       NULL);
+
+        arg->bbox = XtVaCreateManagedWidget("bbox",
+				       asciiTextWidgetClass, browser,
+ 				       XtNeditType, XawtextEdit,
+				       XtNwrap, XawtextWrapNever,
+				       XtNresize, XawtextResizeWidth,
+				       XtNtranslations, trans,
+                                       XtNfromVert, arg->dpi,
+				       XtNvertDistance, 7,
+                                       XtNfromHoriz, arg->bboxleft,
+				       XtNhorizDistance, 3,
+				       XtNwidth, 18,
+				       NULL);
+        sprintf(value, "%d", file_bbox);
+        XtVaSetValues(arg->bbox, XtNstring, value, NULL);
+
+        arg->bboxright = XtVaCreateManagedWidget("xxright",
+				       commandWidgetClass, browser,
+                                       XtNfromVert, arg->dpi,
+				       XtNvertDistance, 7,
+				       XtNfromHoriz, arg->bbox,
+				       XtNhorizDistance, 3,
+				       XtNheight, 20,
+				       XtNwidth, 18,
+				       NULL);
 
         sprintf(value, "%d", Global.numpage);
         XtVaSetValues(arg->page, XtNstring, value, NULL);
+
         XtInstallAccelerators(arg->page, okButton);
         XtInstallAccelerators(arg->dpi, okButton);
         XtInstallAccelerators(arg->zoom, okButton);
         XtInstallAccelerators(arg->alpha, okButton);
-        XtAddCallback(arg->alphalabel, XtNcallback,
+        XtInstallAccelerators(arg->bbox, okButton);
+
+        XtAddCallback(arg->alphaleft, XtNcallback,
+                      alphaCallback, (XtPointer) arg);
+        XtAddCallback(arg->alpharight, XtNcallback,
                       alphaCallback, (XtPointer) arg);
         XtAddCallback(arg->pageLeft, XtNcallback,
                       pageCallback, (XtPointer) arg);
@@ -1506,6 +1608,10 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height)
                       pageCallback, (XtPointer) arg);
         XtAddCallback(arg->pageRight, XtNcallback,
                       pageCallback, (XtPointer) arg);
+        XtAddCallback(arg->bboxleft, XtNcallback, 
+                      bboxCallback, (XtPointer) arg);
+        XtAddCallback(arg->bboxright, XtNcallback, 
+                      bboxCallback, (XtPointer) arg);
         XtAddCallback(arg->refresh, XtNcallback,
                       refreshCallback, (XtPointer) arg);
         XtAddCallback(arg->delete, XtNcallback,
@@ -1757,7 +1863,7 @@ getArg(Widget w, Boolean type, Boolean * built)
 
     switch (type) {
     case 0:
-        shell = buildOpenBrowser(w, cur, 250, 380);
+      shell = buildOpenBrowser(w, cur, 250, 380, 0);
 	break;
     case 1:
         shell = buildSaveBrowser(w, cur);
@@ -1769,7 +1875,7 @@ getArg(Widget w, Boolean type, Boolean * built)
 	shell = buildSimpleBrowser(w, cur, False);
 	break;
     case LOADED:
-        shell = buildOpenBrowser(w, cur, 300, 300);
+        shell = buildOpenBrowser(w, cur, 300, 300, LOADED);
         break;
     }
 
@@ -1794,7 +1900,7 @@ GetFileName(Widget w, int type, char *def, XtCallbackProc okFunc, XtPointer data
     int i, j, im;
 
     if (type == LOADED) {
-        if (!Global.numfiles) return;
+        /* if (!Global.numfiles) return; */
         /* Check whether "loaded files" browser is already popped up */
         for (arg = argList; arg != NULL; arg = arg->next)
 	    if (arg->browserType == LOADED) {
@@ -1856,7 +1962,7 @@ GetFileName(Widget w, int type, char *def, XtCallbackProc okFunc, XtPointer data
     XtPopup(shell, XtGrabNone);
 
     if (arg->bbox)
-        im = &arg->bbox - &arg->browser;
+        im = &arg->bboxright - &arg->browser;
     else
         im = &arg->cancel - &arg->browser;
 
