@@ -87,7 +87,7 @@ static Palette *deltaPal;
  */
 
 #define pixelwrite(x,y)	do {	\
-  mask[y * maskW + x] = True;	\
+  mask[y * maskW + x] = 255;	\
   xMin = MIN(xMin, x);		\
   yMin = MIN(yMin, y);		\
   xMax = MAX(xMax, x);		\
@@ -370,6 +370,13 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     Pixmap pix, gradient;
     Colormap cmap;
 
+    if (event->type == ButtonRelease) {
+        if (Global.popped_up)
+            PopdownMenusGlobal();
+        event->type = None;
+        return;
+    }
+
     if (event->button == Button1)
 	dgc = info->first_gc;
     else if (event->button == Button2)
@@ -424,7 +431,7 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     rect.width = (xMax - xMin) + 1;
     rect.height = (yMax - yMin) + 1;
 
-    pix = MaskDataToPixmap(w, width, height, mask, &rect);
+    pix = AlphaToPixmap(w, width, height, mask, &rect);
 
     XSetClipOrigin(dpy, dgc, rect.x, rect.y);
     XSetClipMask(dpy, dgc, pix);
@@ -482,7 +489,7 @@ FillAdd(Widget w)
     LocalInfo *l = (LocalInfo *) XtMalloc(sizeof(LocalInfo));
     l->done = False;
     l->type = 0;
-    OpAddEventHandler(w, opPixmap, ButtonPressMask, FALSE,
+    OpAddEventHandler(w, opPixmap, ButtonPressMask|ButtonReleaseMask, FALSE,
 		      (OpEventProc) press, l);
     OpAddEventHandler(w, opPixmap, PointerMotionMask, FALSE,
 		      (OpEventProc) motion, l);
@@ -493,7 +500,7 @@ FillAdd(Widget w)
 void 
 FillRemove(Widget w, void *l)
 {
-    OpRemoveEventHandler(w, opPixmap, ButtonPressMask, FALSE,
+    OpRemoveEventHandler(w, opPixmap, ButtonPressMask|ButtonReleaseMask, FALSE,
 			 (OpEventProc) press, l);
     OpRemoveEventHandler(w, opPixmap, PointerMotionMask, FALSE,
 			 (OpEventProc) motion, l);
@@ -521,7 +528,7 @@ GradientFillAdd(Widget w)
     LocalInfo *l = (LocalInfo *) XtMalloc(sizeof(LocalInfo));
     l->done = False;
     l->type = 1;
-    OpAddEventHandler(w, opPixmap, ButtonPressMask, FALSE,
+    OpAddEventHandler(w, opPixmap, ButtonPressMask|ButtonReleaseMask, FALSE,
 		      (OpEventProc) press, l);
     OpAddEventHandler(w, opPixmap, PointerMotionMask, FALSE,
 		      (OpEventProc) motion, l);
@@ -532,7 +539,7 @@ GradientFillAdd(Widget w)
 void 
 GradientFillRemove(Widget w, void *l)
 {
-    OpRemoveEventHandler(w, opPixmap, ButtonPressMask, FALSE,
+    OpRemoveEventHandler(w, opPixmap, ButtonPressMask|ButtonReleaseMask, FALSE,
 			 (OpEventProc) press, l);
     OpRemoveEventHandler(w, opPixmap, PointerMotionMask, FALSE,
 			 (OpEventProc) motion, l);
@@ -569,7 +576,7 @@ FractalFillAdd(Widget w)
     LocalInfo *l = (LocalInfo *) XtMalloc(sizeof(LocalInfo));
     l->done = False;
     l->type = 2;
-    OpAddEventHandler(w, opPixmap, ButtonPressMask, FALSE,
+    OpAddEventHandler(w, opPixmap, ButtonPressMask|ButtonReleaseMask, FALSE,
 		      (OpEventProc) press, l);
     OpAddEventHandler(w, opPixmap, PointerMotionMask, FALSE,
 		      (OpEventProc) motion, l);
@@ -580,7 +587,7 @@ FractalFillAdd(Widget w)
 void 
 FractalFillRemove(Widget w, void *l)
 {
-    OpRemoveEventHandler(w, opPixmap, ButtonPressMask, FALSE,
+    OpRemoveEventHandler(w, opPixmap, ButtonPressMask|ButtonReleaseMask, FALSE,
 			 (OpEventProc) press, l);
     OpRemoveEventHandler(w, opPixmap, PointerMotionMask, FALSE,
 			 (OpEventProc) motion, l);
@@ -760,7 +767,7 @@ buildGradientPixmap(Widget w, OpInfo * info,
     }
 
     pixmap = None;		/* force creation of new pixmap */
-    ImageToPixmapCmap(image, w, &pixmap, cmap);
+    ImageToPixmapCmap(image, w, &pixmap, cmap, 0);
     return pixmap;
 }
 
@@ -788,7 +795,7 @@ buildFractalPixmap(Widget w, OpInfo * info,
 
     XtVaGetValues(w, XtNcolormap, &cmap, NULL);
     pixmap = None;		/* force creation of new pixmap */
-    ImageToPixmapCmap(image, w, &pixmap, cmap);
+    ImageToPixmapCmap(image, w, &pixmap, cmap, 0);
     return pixmap;
 }
 #endif
