@@ -13,10 +13,17 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: image.c,v 1.5 1996/05/31 06:31:44 torsten Exp $ */
+/* $Id: image.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#ifdef __VMS
+#define XtDisplay XTDISPLAY
+#define XtScreen XTSCREEN
+#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#include "Paint.h"
+#include "PaintP.h"
 #include "xpaint.h"
 #include "image.h"
 #include "hash.h"
@@ -37,7 +44,8 @@
 #define ZINDEX8(x, y, img)  ((y) * img->bytes_per_line) + (x)
 #define ZINDEX1(x, y, img)  ((y) * img->bytes_per_line) + ((x) >> 3)
 
-static void imageToPixmapLoop(Display *, Image *, Palette *, XImage *, Pixmap *);
+static void 
+imageToPixmapLoop(Display *, Image *, Palette *, XImage *, Pixmap *, int mode);
 
 /*
  * The functions below are written from X11R5 MIT's code (XImUtil.c)
@@ -85,11 +93,11 @@ ImageNew(int width, int height)
     image->sourcePixmap = None;
     image->sourceMask = None;
     image->scale = 3;
+    image->alpha = NULL;
     if (width == 0 || height == 0)
 	image->data = NULL;
     else
 	image->data = (unsigned char *) XtMalloc(width * height * sizeof(char) * 3);
-    image->maskData = NULL;
     return image;
 }
 
@@ -151,7 +159,7 @@ ImageNewGrey(int width, int height)
 void 
 ImageMakeMask(Image * image)
 {
-    image->maskData = (unsigned char *)
+    image->alpha = (unsigned char *)
 	XtMalloc(image->width * image->height * sizeof(char));
 }
 
@@ -167,11 +175,29 @@ ImageDelete(Image * image)
 	XtFree((XtPointer) image->cmapData);
     if (image->data != NULL)
 	XtFree((XtPointer) image->data);
-    if (image->maskData != NULL)
-	XtFree((XtPointer) image->maskData);
+    if (image->alpha != NULL)
+	XtFree((XtPointer) image->alpha);
     XtFree((XtPointer) image);
 }
 
+/* 
+ * Straightforward image copy
+ */
+void
+ImageCopy(Image * image, Image * output)
+{
+    unsigned char *ip, *op;
+    int x, y;
+
+    for (y = 0; y < image->height; y++) {
+        for (x = 0; x < image->width; x++) {
+            ip = ImagePixel(image, x, y);
+            op = ImagePixel(output, x, y);
+            memcpy(op, ip, 3);
+	}
+    }
+}
+
 /*
 **  Convert a colormap image into a RGB image
 **    useful for writers which only deal with RGB and not
@@ -357,6 +383,17 @@ PixmapToImage(Widget w, Pixmap pix, Colormap cmap)
     image->sourceColormap = (unsigned long) cmap;
     image->sourcePixmap = (unsigned long) pix;
 
+    /*
+    if (w != Global.toplevel) {
+        pw = (PaintWidget) w;
+        if (pw && pw->paint.current.alpha) {
+            x = image->width * image->height;
+            image->alpha = XtMalloc(x);
+            memcpy(image->alpha, pw->paint.current.alpha, x);
+	}
+    }
+    */
+
     if (htable != NULL)
 	HashDestroy(htable);
     XDestroyImage(xim);
@@ -378,7 +415,7 @@ PixmapToImageMask(Widget w, Image * image, Pixmap mask)
     xim = XGetImage(dpy, mask, 0, 0, width, height, AllPlanes, ZPixmap);
 
     ImageMakeMask(image);
-    ip = image->maskData;
+    ip = image->alpha;
 
     if ((endX = image->width) > width)
 	endX = width;
@@ -387,9 +424,9 @@ PixmapToImageMask(Widget w, Image * image, Pixmap mask)
 
     for (y = 0; y < endY; y++) {
 	for (x = 0; x < endX; x++)
-	    *ip++ = (Boolean) XGetPixel(xim, x, y);
+	    *ip++ = (XGetPixel(xim, x, y))? 255:0;
 	for (; x < image->width; x++)
-	    *ip++ = True;
+	    *ip++ = 255;
     }
     XDestroyImage(xim);
 }
@@ -448,8 +485,8 @@ quantizeColormap(Image * input, Palette * map, Boolean flag)
 	}
     }
 
-    output->maskData = input->maskData;
-    input->maskData = NULL;
+    output->alpha = input->alpha;
+    input->alpha = NULL;
     ImageDelete(input);
     return output;
 }
@@ -548,7 +585,8 @@ compressColormap(Image * image)
 **  Side effect	 -- always destroy the input image
  */
 Boolean
-ImageToPixmap(Image * image, Widget w, Pixmap * pix, Colormap * cmap)
+ImageToPixmap(Image * image, Widget w, Pixmap * pix, 
+              Colormap * cmap, int mode)
 {
     Display *dpy = XtDisplay(w);
     Palette *map;
@@ -573,14 +611,14 @@ ImageToPixmap(Image * image, Widget w, Pixmap * pix, Colormap * cmap)
     if (image->cmapSize > 0)
 	compressColormap(image);
 
-    imageToPixmapLoop(dpy, image, map, xim, pix);
+    imageToPixmapLoop(dpy, image, map, xim, pix, mode);
 
     return True;
 }
 
 Pixmap
-MaskDataToPixmap(Widget w, int width, int height,
-		 char *data, XRectangle * rect)
+AlphaToPixmap(Widget w, int width, int height,
+	      char *data, XRectangle * rect)
 {
     Display *dpy = XtDisplay(w);
     GC gc;
@@ -589,6 +627,7 @@ MaskDataToPixmap(Widget w, int width, int height,
     int x, y;
     int xs, ys;
     unsigned char *ucp;
+    unsigned char threshold;
     int pWidth, pHeight;
 
     if (data == NULL)
@@ -625,7 +664,6 @@ MaskDataToPixmap(Widget w, int width, int height,
 		ucp = (unsigned char *) (data + (width * (y + ys)) + xs);
 		for (x = 0; x < pWidth; x++, ucp++) {
 		    unsigned char v = 0x80 >> (x & 7);
-
 		    cp = &op[ZINDEX1(x, y, xim)];
 		    if (*ucp)
 			*cp |= v;
@@ -634,13 +672,13 @@ MaskDataToPixmap(Widget w, int width, int height,
 		}
 	    }
 	} else {
+	    threshold = (unsigned char) Global.alpha_threshold;
 	    for (y = 0; y < pHeight; y++) {
 		ucp = (unsigned char *) (data + (width * (y + ys)) + xs);
 		for (x = 0; x < pWidth; x++, ucp++) {
 		    unsigned char v = 0x01 << (x & 7);
-
 		    cp = &op[ZINDEX1(x, y, xim)];
-		    if (*ucp)
+		    if (*ucp >= threshold)
 			*cp |= v;
 		    else
 			*cp &= ~v;
@@ -661,8 +699,55 @@ MaskDataToPixmap(Widget w, int width, int height,
 Pixmap
 ImageMaskToPixmap(Widget w, Image * image)
 {
-    return MaskDataToPixmap(w, image->width,
-			    image->height, (char *) image->maskData, NULL);
+    return AlphaToPixmap(w, image->width,
+			    image->height, (char *) image->alpha, NULL);
+}
+
+void
+get_color_components(Pixel p, unsigned char *c, int depth)
+{
+    unsigned char u, v;
+
+    if (depth>=24) {
+        c[0] = p>>16;
+        c[1] = (p>>8) & 255;
+        c[2] = p & 255;
+    } else
+    if (depth==16) {
+        u = p>>8;
+	v = p&255;
+        c[0] = ((v>>3)*255)/31;
+        c[1] = ((((v&7)<<3) | (u>>5))*255)/63;
+        c[2] = ((u&31)*255)/31;
+    } else
+    if (depth==15) {
+        u = p>>8;
+	v = p&255;
+        c[0] = ((v>>3)*255)/31;
+        c[1] = ((((v&7)<<3) | (u>>5))*255)/63;
+        c[2] = ((u&31)*255)/31;
+    } else
+        c[0] = p&255;
+}
+
+Pixel
+get_pixel_from_colors(unsigned char *c, int depth)
+{
+    Pixel p;
+    if (depth>=24) {
+        p = ((unsigned int)c[0]<<16) | ((unsigned int)c[1]<<8) | 
+             (unsigned int)c[2];
+    } else
+    if (depth==16) {
+        p = (((c[2]&248)>>3) | ((c[1]&28)<<3)) |
+            ((((c[1]&224)>>5) | (c[0]&248)) << 8);
+    } else
+    if (depth==15) {
+        p = (((c[2]&248)>>3) | ((c[1]&56)<<2)) |
+            ((((c[1]&192)>>6) | ((c[0]&248)>>1)) << 8);
+    } else
+        p = c[0]||c[1]||c[2];
+    return p;
 }
 
 /*
@@ -673,39 +758,149 @@ ImageMaskToPixmap(Widget w, Image * image)
  */
 static void 
 imageToPixmapLoop(Display * dpy, Image * image,
-		  Palette * map, XImage * xim, Pixmap * pix)
+		  Palette * map, XImage * xim, Pixmap * pix, int mode)
 {
     GC gc;
-    int x, y;
+    int x, y, twist, hide, cmapsize;
     int width = image->width, height = image->height;
+    unsigned char bg_r, bg_g, bg_b, gray;
+    Pixel p;
+
+    if (!image->alpha) mode = 0;
+
+    hide = mode & 4;
+    twist = mode & 3;
+
+    if (twist == 3) {
+        bg_r = (Global.background>>16)&255;
+        bg_g = (Global.background>>8)&255;
+        bg_b = (Global.background)&255;
+    }
 
     if (image->cmapSize > 0) {
 	unsigned short *sdp = (unsigned short *) image->data;
 	unsigned char *cdp = (unsigned char *) image->data;
-	Pixel *list = (Pixel *) XtCalloc(sizeof(Pixel), image->cmapSize);
-	XColor *xcol = (XColor *) XtCalloc(sizeof(XColor), image->cmapSize);
+        unsigned char *cg = image->alpha;
+	Pixel *list;
+	XColor *xcol; 
 
-	for (y = 0; y < image->cmapSize; y++) {
-	    xcol[y].red = image->cmapData[y * 3 + 0] << 8;
-	    xcol[y].green = image->cmapData[y * 3 + 1] << 8;
-	    xcol[y].blue = image->cmapData[y * 3 + 2] << 8;
+        if (twist == 1) 
+            cmapsize = 256;
+        else
+            cmapsize = image->cmapSize + 2;
+
+        list = (Pixel *) XtCalloc(sizeof(Pixel), cmapsize);
+        xcol = (XColor *) XtCalloc(sizeof(XColor), cmapsize);
+
+        if (twist == 1) {
+	    for (y = 0; y < 256; y++) {
+	        xcol[y].red = y << 8;
+	        xcol[y].green = y << 8;
+	        xcol[y].blue = y << 8;
+	    }
+	} else {
+	    for (y = 0; y < image->cmapSize; y++) {
+	        xcol[y].red = image->cmapData[y * 3 + 0] << 8;
+	        xcol[y].green = image->cmapData[y * 3 + 1] << 8;
+	        xcol[y].blue = image->cmapData[y * 3 + 2] << 8;
+	    }
 	}
 
-	PaletteAllocN(map, xcol, image->cmapSize, list);
+        if (twist == 2) {
+	    y = image->cmapSize;
+	    xcol[y].red = xcol[y].green = xcol[y].blue = 153 << 8;
+            ++y;
+	    xcol[y].red = xcol[y].green = xcol[y].blue = 102 << 8;
+	}
+
+        if (twist == 3) {
+	    y = image->cmapSize;
+	    xcol[y].red = bg_r << 8;
+	    xcol[y].green = bg_g << 8;
+	    xcol[y].blue = bg_b << 8;
+	}
+
+	PaletteAllocN(map, xcol, cmapsize, list);
 
 	if (xim->bits_per_pixel == 8) {
 	    unsigned char *data = (unsigned char *) xim->data;
 
-	    for (y = 0; y < height; y++)
-		for (x = 0; x < width; x++, sdp++, cdp++)
-		    data[ZINDEX8(x, y, xim)] = list[image->cmapSize > 256 ? *sdp : *cdp];
-	} else {
+            if (twist == 1)
+	        for (y = 0; y < height; y++)
+		for (x = 0; x < width; x++, cg++)
+		    data[ZINDEX8(x, y, xim)] = list[*cg];
+            else
+            if (image->cmapSize > 256)
+	        for (y = 0; y < height; y++)
+		for (x = 0; x < width; x++, sdp++)
+		    data[ZINDEX8(x, y, xim)] = list[*sdp];
+	    else
+	        for (y = 0; y < height; y++)
+		for (x = 0; x < width; x++, cdp++)
+		    data[ZINDEX8(x, y, xim)] = list[*cdp];
+
+            if (twist == 2)
+	        for (y = 0; y < height; y++)
+		for (x = 0; x < width; x++, cg++)
+		    if (*cg<(unsigned char)Global.alpha_threshold)
+                        data[ZINDEX8(x, y, xim)] = 
+			    list[image->cmapSize+((x/12+y/12)&1)];
+            else
+            if (twist == 3)
+	        for (y = 0; y < height; y++)
+		for (x = 0; x < width; x++, cg++)
+		    if (*cg<(unsigned char)Global.alpha_threshold)
+                        data[ZINDEX8(x, y, xim)] = list[image->cmapSize];
+	} else 
+	if (twist == 0) {
 	    /*
 	    **	Slow loop
 	     */
+	    if (image->cmapSize > 256) {
+	        for (y = 0; y < height; y++) {
+		    for (x = 0; x < width; x++, sdp++)
+		        XPutPixel(xim, x, y, list[*sdp]);
+		    if (y % 256 == 0)
+		        StateTimeStep();
+		}
+	    } else {
+	        for (y = 0; y < height; y++) {
+		    for (x = 0; x < width; x++, cdp++)
+		        XPutPixel(xim, x, y, list[*cdp]);
+		    if (y % 256 == 0)
+		        StateTimeStep();
+		}
+	    }
+	} else {
+	    unsigned char rgb[3];
+            unsigned char *cg = image->alpha;
+
 	    for (y = 0; y < height; y++) {
-		for (x = 0; x < width; x++, sdp++, cdp++)
-		    XPutPixel(xim, x, y, list[image->cmapSize > 256 ? *sdp : *cdp]);
+	        for (x = 0; x < width; x++, cdp++, sdp++) {
+                     if (twist == 1) {
+		         *rgb = *cg++;
+                         rgb[1] = rgb[2] = *rgb;
+                         p = get_pixel_from_colors(rgb, xim->bits_per_pixel);
+		     } else {
+		         if (image->cmapSize > 256)
+		             p = list[*sdp];
+                         else
+		             p = list[*cdp];
+                         gray = *cg++;
+                         get_color_components(p, rgb, xim->bits_per_pixel);
+                         if (twist == 2) {
+		             if ((x/12+y/12)&1)
+		                 bg_r = bg_g = bg_b = 153;
+                             else
+		                 bg_r = bg_g = bg_b = 102;
+			 }
+                         rgb[0] = (rgb[0]*gray+bg_r*(255-gray))/255;
+                         rgb[1] = (rgb[1]*gray+bg_g*(255-gray))/255;
+                         rgb[2] = (rgb[2]*gray+bg_b*(255-gray))/255;
+                         p = get_pixel_from_colors(rgb, xim->bits_per_pixel);
+		     }
+                     XPutPixel(xim, x, y, p);
+		}
 		if (y % 256 == 0)
 		    StateTimeStep();
 	    }
@@ -716,15 +911,37 @@ imageToPixmapLoop(Display * dpy, Image * image,
     } else {
 	int step = 64 * 256 / width;
 	unsigned char *cp = image->data;
+        unsigned char *cg = image->alpha;
 
 	for (y = 0; y < height; y++) {
 	    for (x = 0; x < width; x++) {
 		XColor c;
 		Pixel p;
 
-		c.red = *cp++ << 8;
-		c.green = *cp++ << 8;
-		c.blue = *cp++ << 8;
+                if (twist == 0) {
+		    c.red = *cp++ << 8;
+		    c.green = *cp++ << 8;
+		    c.blue = *cp++ << 8;
+		} else
+                if (twist == 1) {
+		    c.red = *cg++ << 8;
+                    c.green = c.red;
+                    c.blue = c.red;
+		} else {
+		    if (twist == 2) {
+		        if ((x/12+y/12)&1)
+		            bg_r = bg_g = bg_b = 153;
+                        else
+		            bg_r = bg_g = bg_b = 102;
+		    }
+		    gray = *cg++;
+                    c.red = *cp++;
+                    c.green = *cp++;
+                    c.blue = *cp++;
+                    c.red = ((c.red*gray+bg_r*(255-gray))/255) << 8;
+                    c.green = ((c.green*gray+bg_g*(255-gray))/255) << 8;
+                    c.blue = ((c.blue*gray+bg_b*(255-gray))/255) << 8;
+		}
 
 		p = PaletteAlloc(map, &c);
 
@@ -745,6 +962,7 @@ imageToPixmapLoop(Display * dpy, Image * image,
 
     XDestroyImage(xim);
 
+    if (hide) image->alpha = NULL;
     ImageDelete(image);
 }
 
@@ -754,7 +972,8 @@ imageToPixmapLoop(Display * dpy, Image * image,
  * Unless pix == None, a new pixmap is created.
  */
 Boolean
-ImageToPixmapCmap(Image * image, Widget w, Pixmap * pix, Colormap cmap)
+ImageToPixmapCmap(Image * image, Widget w, Pixmap * pix, 
+                  Colormap cmap, int mode)
 {
     Display *dpy = XtDisplay(w);
     Palette *map;
@@ -780,7 +999,7 @@ ImageToPixmapCmap(Image * image, Widget w, Pixmap * pix, Colormap cmap)
     if (image->cmapSize > 0)
 	compressColormap(image);
 
-    imageToPixmapLoop(dpy, image, map, xim, pix);
+    imageToPixmapLoop(dpy, image, map, xim, pix, mode);
 
     return True;
 }
