@@ -97,6 +97,8 @@ struct imageprocessinfo {
 
 #endif
 
+#define ImageAlphaPixel(image, x, y) ((image)->alpha+((x)+(y)*(image)->width))
+
 #define ImagePixel(image, x, y)						\
 	(((image)->cmapSize == 0)					\
 	  ? &((image)->data[((y) * (image)->width + (x)) * 3])		\
@@ -167,6 +169,7 @@ Image *ImageNormContrast(Image * input);
 Image *ImageHistogram(Image * input);
 Image *ImageSolarize(Image * input);
 Image *ImageGrey(Image * input);
+Image *ImageBWMask(Image * input);
 Image *ImageModifyRGB(Image * input);
 Image *ImageQuantize(Image * input);
 Image *ImageUserDefined(Image * input);
