@@ -425,8 +425,8 @@ ImageCompress(Image * input, int ncolors, int noforce)
 
 	HashDestroy(htable);
 	XtFree((XtPointer) ctable);
-	output->maskData = input->maskData;
-	input->maskData = NULL;
+	output->alpha = input->alpha;
+	input->alpha = NULL;
 	ImageDelete(input);
 	outputImage = output;
 	return output;
@@ -506,8 +506,8 @@ ImageCompress(Image * input, int ncolors, int noforce)
     }
 
     output = ImageNewCmap(input->width, input->height, allocated);
-    output->maskData = input->maskData;
-    input->maskData = NULL;
+    output->alpha = input->alpha;
+    input->alpha = NULL;
 
     for (i = 0; i < allocated; i++) {
 	long red, green, blue;
