@@ -63,6 +63,9 @@ void *dl_image = NULL;
 **  Some real image processing
  */
 
+extern int file_isSpecialImage;
+extern int file_transparent;
+
 extern struct imageprocessinfo ImgProcessInfo;
 extern FILE * openTempFile(char **np);
 extern void removeTempFile(void);
@@ -1462,6 +1465,30 @@ ImageGrey(Image * input)
     return output;
 }
 
+Image *
+ImageBWMask(Image * input)
+{
+    Image *output = ImageNew(input->width, input->height);
+    unsigned char *p, *op = output->data;
+    unsigned char bg[3];
+    int x, y, v;
+
+    bg[0] = (Global.background>>16)&255;
+    bg[1] = (Global.background>>8)&255;
+    bg[2] = (Global.background)&255;
+
+    for (y = 0; y < input->height; y++) {
+	for (x = 0; x < input->width; x++) {
+	    p = ImagePixel(input, x, y);
+            if (memcmp(p, bg, 3)) v = 0xff; else v = 0;
+	    *op++ = v;
+	    *op++ = v;
+	    *op++ = v;
+	}
+    }
+    return output;
+}
+
 /*
  * Modify RGB values of an image.
  */
@@ -1685,6 +1712,7 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
    Visual *visual;
    Pixmap pix;
    Colormap cmap;
+   Widget paint;
    char *tmp, *ptr, *text;
    char cmd[512];
    char header[30];
@@ -1790,8 +1818,16 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
          Notice(Global.toplevel, msgText[UNABLE_TO_CREATE_IMAGE]);
 	 return;
       }
-      graphicCreate(makeGraphicShell(Global.toplevel), 0, 0, -1, 
-                    pix, cmap, alpha);
+      if (alpha) {
+         file_isSpecialImage = 1;
+         file_transparent = 1;
+      }
+      paint = graphicCreate(makeGraphicShell(Global.toplevel), 0, 0, -1, 
+                            pix, cmap, alpha);
+      SetEditable((Widget)paint,
+                            !(alpha && Global.alpha_mode>=2),
+                            (alpha!=NULL && Global.alpha_mode==0));
+      free(alpha);
       return;
    }
 
