@@ -18,11 +18,14 @@
 #define XtDisplay XTDISPLAY
 #endif
 
+#include <X11/Xlib.h>
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <math.h>
 #include <stdlib.h>
 #include "xpaint.h"
+#include "Paint.h"
+#include "PaintP.h"
 #include "misc.h"
 #include "Paint.h"
 #include "ops.h"
@@ -35,7 +38,7 @@ static double headAngle = 25.0;
 
 typedef struct {
     int type;
-    Boolean didUndo, didSet, headArrow, tracking;
+    Boolean didUndo, headArrow, tracking;
     int rx, ry;
     int startX, startY, endX, endY;
     int drawn;
@@ -43,19 +46,19 @@ typedef struct {
     GC gcx;
 } LocalInfo;
 
-static void 
+static void
 DrawArrow (Widget w, Drawable d, GC gc, int stX,int stY, int enX, int enY,
-           Boolean headArrow)
+           Boolean headArrow, Boolean dozoom)
 {
   XPoint xpoints[4];
   Display *dpy; 
   int lw;
-  int i, l, m, ux, uy, lx=0, ly=0, mx=0, my=0;
+  int i, ux, uy, lx=0, ly=0, mx=0, my=0, z;
   int DeltaX, DeltaY;
-  float norm;
+  float norm, ll, mm;
 
   dpy = XtDisplay(w);
-  XtVaGetValues(w, XtNlineWidth, &lw, NULL);
+  XtVaGetValues(w, XtNlineWidth, &lw, XtNzoom, &z, NULL);
   ux = uy = 0;
 
   if (withArrow || headArrow) {
@@ -64,15 +67,21 @@ DrawArrow (Widget w, Drawable d, GC gc, int stX,int stY, int enX, int enY,
     norm = sqrt(pow(DeltaX, 2) + pow(DeltaY, 2)) + 0.001;
    
     if ( lw <= 2 || headArrow)
-       l = headSize;
+      ll = (float) headSize;
     else
-       l = 4*(lw-1)+headSize;
-    m = l * tan(headAngle*M_PI/180.0);
+      ll = (float) (4*(lw-1)+headSize);
+
+    if (dozoom) {
+       if (z>0) ll = ll * (float) z;
+       if (z<0) ll = ll / (float)(-z);
+    }
+
+    mm = ll * tan(headAngle*M_PI/180.0) +0.5;
    
-    lx = (int) (0.49 + (float) (abs(DeltaX) * l) / norm);
-    ly = (int) (0.49 + (float) (abs(DeltaY) * l) / norm);
-    mx = (int) (0.49 + (float) (abs(DeltaY) * m) / norm);
-    my = (int) (0.49 + (float) (abs(DeltaX) * m) / norm);
+    lx = (int) (0.49 + (float) (abs(DeltaX) * ll) / norm);
+    ly = (int) (0.49 + (float) (abs(DeltaY) * ll) / norm);
+    mx = (int) (0.49 + (float) (abs(DeltaY) * mm) / norm);
+    my = (int) (0.49 + (float) (abs(DeltaX) * mm) / norm);
     my = -my;
    
     if (DeltaX < 0) {
@@ -109,8 +118,9 @@ DrawArrow (Widget w, Drawable d, GC gc, int stX,int stY, int enX, int enY,
        xpoints[1].y = xpoints[0].y - ly + my;
        xpoints[2].x = xpoints[0].x - lx - mx;
        xpoints[2].y = xpoints[0].y - ly - my;
-       XFillPolygon(dpy, d, gc, xpoints, 3, Complex, CoordModeOrigin);
-    }
+       z = 3;
+    } else
+       z = 4;
     if (headType == 2) {
        xpoints[1].x = xpoints[0].x - lx + mx;
        xpoints[1].y = xpoints[0].y - ly + my;
@@ -118,7 +128,6 @@ DrawArrow (Widget w, Drawable d, GC gc, int stX,int stY, int enX, int enY,
        xpoints[2].y = xpoints[0].y - ly/2;
        xpoints[3].x = xpoints[0].x - lx - mx;
        xpoints[3].y = xpoints[0].y - ly - my;
-       XFillPolygon(dpy, d, gc, xpoints, 4, Complex, CoordModeOrigin);
     }
     if (headType == 3) {
        xpoints[1].x = xpoints[0].x - lx + mx;
@@ -128,14 +137,18 @@ DrawArrow (Widget w, Drawable d, GC gc, int stX,int stY, int enX, int enY,
        xpoints[3].x = xpoints[0].x - lx - mx;
        xpoints[3].y = xpoints[0].y - ly - my;
        XFillPolygon(dpy, d, gc, xpoints, 4, Complex, CoordModeOrigin);
+       if (!dozoom)
+	   for (i=0; i<4; i++) UndoGrow(w, (int)xpoints[i].x,(int)xpoints[i].y);
        lx /= 3;
        ly /= 3;
        for (i=0; i<4; i++) {
            xpoints[i].x -= lx;
            xpoints[i].y -= ly;
        }
-       XFillPolygon(dpy, d, gc, xpoints, 4, Complex, CoordModeOrigin);
     }
+    XFillPolygon(dpy, d, gc, xpoints, z, Complex, CoordModeOrigin);
+    if (!dozoom)
+       for (i=0; i<z; i++) UndoGrow(w, (int)xpoints[i].x, (int)xpoints[i].y);
   }
 }
  
@@ -162,12 +175,11 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 	l->drawable = info->drawable;
 	l->drawn = False;
 	l->didUndo = False;
-	l->didSet = False;
 	l->tracking = True;
     } else if (event->button == Button2) {
 	if (l->drawn)
 	    DrawArrow(w, info->drawable, l->gcx,
-		      l->startX, l->startY, l->endX, l->endY, l->headArrow);
+		      l->startX, l->startY, l->endX, l->endY, l->headArrow, 1);
 	l->tracking = False;
 	l->drawn = False;
         return;
@@ -175,13 +187,13 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     if (l->tracking) {
 	if (l->drawn)
 	    DrawArrow(w, info->drawable, l->gcx,
-		      l->startX, l->startY, l->endX, l->endY, l->headArrow);
+		      l->startX, l->startY, l->endX, l->endY, l->headArrow, 1);
 
 	l->endX = event->x;
 	l->endY = event->y;
 
 	DrawArrow(w, info->drawable, l->gcx,
-		  l->startX, l->startY, l->endX, l->endY, l->headArrow);
+		  l->startX, l->startY, l->endX, l->endY, l->headArrow, 1);
 	l->drawn = True;
     }
 }
@@ -195,7 +207,7 @@ motion(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
 
     if (l->drawn)
 	DrawArrow(w, info->drawable, l->gcx,
-		  l->startX, l->startY, l->endX, l->endY, l->headArrow);
+		  l->startX, l->startY, l->endX, l->endY, l->headArrow, 1);
 
     l->type = (event->state & ShiftMask);
 
@@ -236,13 +248,13 @@ motion(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
      */
     if ((l->drawn = (l->startX != l->endX || l->startY != l->endY)))
 	DrawArrow(w, info->drawable, l->gcx,
-		  l->startX, l->startY, l->endX, l->endY, l->headArrow);
+		  l->startX, l->startY, l->endX, l->endY, l->headArrow, 1);
 }
 
 static void 
 release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 {
-    XRectangle undo;
+    XRectangle *undo = NULL;
     int mask;
 
     if (!l->tracking && ((info->surface == opWindow) || l->didUndo))
@@ -275,7 +287,7 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 
     if (l->drawn && info->surface == opWindow) {
 	DrawArrow(w, info->drawable, l->gcx,
-		  l->startX, l->startY, l->endX, l->endY, l->headArrow);
+		  l->startX, l->startY, l->endX, l->endY, l->headArrow, 1);
 	l->drawn = False;
         if (Global.escape) return;
     }
@@ -290,9 +302,16 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 	return;
 
     if (!l->didUndo && info->surface == opPixmap) {
-	UndoStart(w, info);
+        PaintWidget pw = (PaintWidget) w;
+        UndoStart(w, info);
+	undo = &pw->paint.undo->box;
+	undo->x = l->rx;
+        undo->y = l->ry;
+        undo->width = pw->paint.lineWidth * 2 + 1;
+        undo->height = undo->width;
 	l->didUndo = True;
     }
+
     if (l->type) {
 	int ax = ABS(event->x - l->rx);
 	int ay = ABS(event->y - l->ry);
@@ -325,21 +344,16 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 	l->endY = event->y;
     }
 
-    SetCapAndJoin(w, info->first_gc, JoinMiter);
+    SetCapAndJoin(w, info->first_gc, 
+                  ((Global.cap)?Global.cap-1:CapButt),
+                  ((Global.join)?Global.join-1:JoinMiter));
     DrawArrow(w, info->drawable, info->first_gc,
-	      l->rx, l->ry, l->endX, l->endY, l->headArrow);
+	             l->rx, l->ry, l->endX, l->endY, l->headArrow, 0);
 
     if (info->surface == opPixmap) {
-	XYtoRECT(l->rx, l->ry, l->endX, l->endY, &undo);
-
-	if (l->didSet) {
-	    UndoGrow(w, l->endY, l->endY);
-	} else {
-	    UndoSetRectangle(w, &undo);
-	    l->didSet = True;
-	}
-
-	PwUpdate(w, &undo, False);
+	UndoGrow(w, l->rx, l->ry);
+	UndoGrow(w, l->endX, l->endY);
+	PwUpdate(w, undo, False);
     }
 
     if (!multiLine || l->headArrow)
@@ -384,7 +398,7 @@ LineRemove(Widget w, void *p)
 
     if (multiLine && l->tracking && l->drawn)
 	DrawArrow(w, l->drawable, l->gcx,
-		  l->startX, l->startY, l->endX, l->endY, l->headArrow);
+		  l->startX, l->startY, l->endX, l->endY, l->headArrow, 1);
 
     XtFree((XtPointer) l);
 }
@@ -424,7 +438,7 @@ ArrowRemove(Widget w, void *p)
 
     if (multiLine && l->tracking && l->drawn)
 	DrawArrow(w, l->drawable, l->gcx,
-		  l->startX, l->startY, l->endX, l->endY, l->headArrow);
+		  l->startX, l->startY, l->endX, l->endY, l->headArrow, 1);
 
     XtFree((XtPointer) l);
 }
@@ -460,9 +474,9 @@ DashSetStyle(char *dashstyle)
     char c;
 
     len = strlen(dashstyle);
-    if (len > 48) {
-        dashstyle[48] = '\0';
-	len = 48;
+    if (len > 64) {
+        dashstyle[64] = '\0';
+	len = 64;
     }
     Global.dashoffset = 0;
     c = dashstyle[0];
