@@ -37,6 +37,7 @@
 #include <X11/Shell.h>
 #include <X11/StringDefs.h>
 #include <X11/xpm.h>
+#include <X11/Xft/Xft.h>
 
 #define DEFINE_GLOBAL
 #include "xpaint.h"
@@ -60,6 +61,7 @@ char lang[10];
 char *routine;
 
 extern int magnifier_closing_down;
+extern char *fontNames[];
 
 extern void BrushInit(Widget toplevel);
 extern int TestScriptC(char *file);
@@ -810,6 +812,8 @@ main(int argc, char *argv[])
 			       options, XtNumber(options), &argc, argv,
 			       appDefaults, args, nargs);
 
+    dpy = Global.display = XtDisplay(toplevel);
+
     memset(&appInfo, 0, sizeof(AppInfo));
     XtGetApplicationResources(toplevel, (XtPointer) &appInfo,
 			      resources, XtNumber(resources), NULL, 0);
@@ -856,6 +860,10 @@ main(int argc, char *argv[])
     Global.alpha_default = 255;
     Global.alpha_threshold = 128;
     Global.background = 0xffffff;
+    /* select by default Times Regular at 18 pt */
+    Global.xft_name = NULL;
+    Global.xft_font = NULL;
+    setDefaultGlobalFont(dpy, fontNames[1]);
 
     /*
     **
@@ -912,7 +920,6 @@ main(int argc, char *argv[])
     Global.explore = False;
 
     Global.appContext = XtWidgetToApplicationContext(toplevel);
-    dpy = Global.display = XtDisplay(toplevel);
 
     /*
     **  Call the initializers
