@@ -166,6 +166,20 @@ destroy_right(Widget w, XtPointer data, XtPointer junk)
     }
 }
 
+/*
+int
+getTransitionalMenu()
+{
+    return transitional;
+}
+
+void
+setTransitionalMenu()
+{
+    transitional = 1;
+}
+*/
+
 void
 PopdownMenusGlobal()
 {
