@@ -48,6 +48,48 @@
 #include "palette.h"
 #include "protocol.h"
 
+/* 
+ *  Set min size for WM hints
+ */
+
+char *
+ZoomToStr(int zoom)
+{
+    static char buffer[20]; 
+    if (zoom == -1) zoom = 1;
+    if (zoom>0)
+        sprintf(buffer, "%d",  zoom);
+    else
+        sprintf(buffer, ":%d", -zoom);
+    return buffer;
+}
+
+int
+StrToZoom(char *str)
+{
+    int zoom;
+    char *ptr;
+    if ((ptr=strchr(str,'/')))
+	zoom = -atoi(ptr+1);
+    else
+    if ((ptr=strchr(str,':')))
+	zoom = -atoi(ptr+1);
+    else
+	zoom = atoi(str);
+    if (zoom == -1) zoom = 1;
+    return zoom;
+}
+
+void
+XtSetMinSizeHints(Widget w, int u, int v)
+{
+    XSizeHints xsh;
+    xsh.min_width = u;
+    xsh.min_height = v;
+    xsh.flags = PMinSize;
+    XSetNormalHints(XtDisplay(w), XtWindow(w), &xsh);
+}
+
 /* 
  *  X error handler for,  mainly for future use
  */
@@ -61,10 +103,11 @@ privateXErrorHandler(Display *dpy, XErrorEvent *myerr)
      * the pixmap.  Tell the user this.
      */
     if (myerr->error_code == None) return 0;
+    if (!strcmp(routine, "snapshot")) return 0;
     fprintf(stderr,"%s:\n", routine); 
     if (myerr->error_code == BadAlloc) {
-        fprintf(stderr, msgText[INSUFFICIENT_RESOURCES_FOR_OPERATION]);
-	fprintf(stderr, msgText[PROBABLY_NOT_ENOUGH_MEMORY_TO_ALLOCATE_PIXMAP]);
+        fprintf(stderr, "%s", msgText[INSUFFICIENT_RESOURCES_FOR_OPERATION]);
+	fprintf(stderr, "%s", msgText[PROBABLY_NOT_ENOUGH_MEMORY_TO_ALLOCATE_PIXMAP]);
     }
 
     /* Now print out the error string that X generates */
@@ -89,7 +132,7 @@ GetToplevel(Widget w)
 Widget
 GetShell(Widget w)
 {
-    while (!XtIsShell(w))
+    while (w && !XtIsShell(w))
 	w = XtParent(w);
     return w;
 }
@@ -115,17 +158,16 @@ SetPencilCursor(Widget w)
 }
 
 void
-SetCapAndJoin(Widget w, GC gc, int join)
+SetCapAndJoin(Widget w, GC gc, int cap, int join)
 {
     int lw;
     XtVaGetValues(w, XtNlineWidth, &lw, NULL);
     if (Global.dashnumber == 0) {
-        XSetLineAttributes(XtDisplay(w), gc, lw, LineSolid, CapRound, join);
+        XSetLineAttributes(XtDisplay(w), gc, lw, LineSolid, cap, join);
     } else {
         XSetDashes(XtDisplay(w), gc, Global.dashoffset, 
                            Global.dashlist, Global.dashnumber);
-        XSetLineAttributes(XtDisplay(w), gc, lw, LineOnOffDash, 
-                           CapButt, join);
+        XSetLineAttributes(XtDisplay(w), gc, lw, LineOnOffDash, cap, join);
     }
 }
 
@@ -224,7 +266,7 @@ static BackgroundList *bgList = NULL;
 Pixmap
 GetBackgroundPixmap(Widget w)
 {
-    Widget p, n;
+    Widget p, n = None;
     Display *dpy = XtDisplay(w);
     int depth;
     Pixmap pix;
@@ -232,6 +274,7 @@ GetBackgroundPixmap(Widget w)
     BackgroundList *cur;
     Widget tw;
 
+    if (!w) return None;
     for (n = XtParent(p = w); !XtIsShell(n); n = XtParent(p = n));
 
     XtVaGetValues(p, XtNdepth, &depth, NULL);
@@ -429,7 +472,7 @@ xmalloc(size_t n)
     if ((p = malloc(n)) != NULL)
 	return p;
     if (msgText)
-        fprintf(stderr, msgText[OUT_OF_MEMORY]);
+        fprintf(stderr, "%s", msgText[OUT_OF_MEMORY]);
     else
         fprintf(stderr, "Out of memory !\n");
     exit(1);
@@ -458,6 +501,7 @@ AutoCrop(Widget paint)
     int x, y, width, height, top, bottom, left, right;
     Pixmap pix;
     Pixel corner[4], bg;
+    unsigned char *alpha;
     XImage *xim;
     Colormap cmap;
     Palette *palette;
@@ -541,6 +585,18 @@ AutoCrop(Widget paint)
 	StateSetBusy(False);
 	return;
     }
+
+
+    if (pw->paint.current.alpha) {
+        i = rect.width * rect.height;
+        alpha = (unsigned char *)XtMalloc(i);
+        for (y=0; y<rect.height; y++)
+	    memcpy(alpha+y*rect.width,
+                   pw->paint.current.alpha+(rect.x+(y+rect.y)*width), 
+                   rect.width);
+    } else
+        alpha = NULL;
+
     PwRegionSet(paint, &rect, None, None);
     PwRegionGet(paint, &pix, None);
     PwRegionFinish(paint, True);
@@ -550,6 +606,8 @@ AutoCrop(Widget paint)
 
     pw->paint.dirty = True;
     PwUpdateDrawable((Widget) paint, XtWindow(paint), NULL);
+    pw->paint.current.alpha = alpha;
+
     StateSetBusy(False);
 }
 
@@ -654,7 +712,7 @@ void
 mousewheelScroll(Widget w, void * l, XEvent * event, Boolean * flg)
 {
     Widget sb;
-    int call_data;
+    long call_data;
     if (event->xbutton.button == 4 || event->xbutton.button == 5) {
         if (l==NULL) {
 	    sb = ((TextWidget)w)->text.vbar;
@@ -662,16 +720,20 @@ mousewheelScroll(Widget w, void * l, XEvent * event, Boolean * flg)
         else
         if (l==(void *)1) {
 	    w = XtParent(w);
+            if (!w) return;
 	    sb = XtNameToWidget(w, "vertical");	   
         } 
         else
         if (l==(void *)2) {
-	    w = XtParent(XtParent(w));
+	    w = XtParent(w);
+            if (!w) return;
+	    w = XtParent(w);
+            if (!w) return;
 	    sb = XtNameToWidget(w, "vertical");	   
 	}
         else return;
         if (sb == None) return;
-#ifdef XAW3DG
+#ifdef XAW_ARROW_SCROLLBARS
         if (((ScrollbarWidget)sb)->scrollbar.scroll_mode == 2 /* if scroll continuous */
             || LookAhead (sb, event))
             return;
