@@ -90,6 +90,7 @@ extern void GetFileName(Widget w, int type, char *def,
 
 /* fontSelect.c */
 extern void FontSelect(Widget w, Widget paint);
+extern void * setDefaultGlobalFont(Display *dpy, char *name);
 
 /* iprocess.c */
 extern Boolean isFilterDefined();
@@ -178,7 +179,7 @@ extern void changeBackground(Widget w, XtPointer paintArg, XtPointer junk2);
 extern void AddItemToCanvasPalette(Widget paint, Pixel p, Pixmap pix);
 extern Boolean inCanvasPixmaps(Widget paint, Pixmap pix);
 extern void AddFileToGlobalList(char * file);
-extern void RemoveFileToGlobalList(char * file);
+extern void RemoveFileFromGlobalList(char * file);
 
 /* pattern.c */
 extern void PatternEdit(Widget w, Pixel *pixels,
