@@ -1236,6 +1236,7 @@ OperationSet(String names[], int num)
         match = &iconList[i];
         if (CurrentOp == match->data) return;
 	XawToggleSetCurrent(iconListWidget, (XtPointer) match->name);
+        CurrentOp = match->data;
         return;
     }
 
@@ -1249,8 +1250,10 @@ OperationSet(String names[], int num)
 	    }
 	}
     }
-    if (match != NULL)
+    if (match != NULL) {
 	XawToggleSetCurrent(iconListWidget, (XtPointer) match->name);
+        CurrentOp = match->data;
+    }
 }
 
 #if 0
