@@ -13,21 +13,24 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: pattern.c,v 1.9 2000/09/02 08:26:20 torsten Exp $ */
+/* $Id: pattern.c,v 1.18 2005/03/20 20:15:32 demailly Exp $ */
+
+#include <stdio.h>
 
 #include <X11/IntrinsicP.h>
 #include <X11/Shell.h>
 #include <X11/StringDefs.h>
-#include <X11/Xaw/Box.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Scrollbar.h>
-#include <X11/Xaw/Viewport.h>
-#include <X11/Xaw/Command.h>
-#include <X11/Xaw/Paned.h>
 #include <X11/CoreP.h>
-#include <X11/Xaw/ToggleP.h>
 #include <X11/cursorfont.h>
-#include <stdio.h>
+
+#include "xaw_incdir/Box.h"
+#include "xaw_incdir/Form.h"
+#include "xaw_incdir/Scrollbar.h"
+#include "xaw_incdir/Viewport.h"
+#include "xaw_incdir/Command.h"
+#include "xaw_incdir/Paned.h"
+#include "xaw_incdir/ToggleP.h"
+
 #include "Colormap.h"
 #include "Paint.h"
 #include "palette.h"
@@ -50,6 +53,11 @@
 #include <unistd.h>
 #endif
 
+#ifdef XAW3D
+#define BSP 4
+#define BWD 0
+#define COLSP 4
+#else
 #ifdef XAW95
 #define BSP 6
 #define BWD 0
@@ -59,9 +67,13 @@
 #define BWD 1
 #define COLSP 4
 #endif
+#endif
 
-extern 
-int WriteAsciiPNMfd(FILE * fd, Image * image);
+extern int
+WriteAsciiPNMfd(FILE * fd, Image * image);
+
+extern Widget
+createBrushDialog(Widget w);
 
 static PaintMenuItem paletteMenu[] =
 {
@@ -100,15 +112,14 @@ static PaintMenuItem editMenu[] =
 
 static PaintMenuItem lineMenu[] =
 {
-    MI_FLAGCB("1", MF_CHECK | MF_GROUP3, lineWidth, NULL),
-    MI_FLAGCB("2", MF_CHECK | MF_GROUP3, lineWidth, NULL),
-    MI_FLAGCB("4", MF_CHECK | MF_GROUP3, lineWidth, NULL),
-    MI_FLAGCB("6", MF_CHECK | MF_GROUP3, lineWidth, NULL),
-    MI_FLAGCB("8", MF_CHECK | MF_GROUP3, lineWidth, NULL),
-#define LW_GENERAL 5
-    MI_FLAGCB("select", MF_CHECK | MF_GROUP3, lineWidth, NULL),
+    MI_FLAGCB("1", MF_CHECK | MF_GROUP3, lineStyleCallback, NULL),
+    MI_FLAGCB("2", MF_CHECK | MF_GROUP3, lineStyleCallback, NULL),
+    MI_FLAGCB("4", MF_CHECK | MF_GROUP3, lineStyleCallback, NULL),
+    MI_FLAGCB("6", MF_CHECK | MF_GROUP3, lineStyleCallback, NULL),
+    MI_FLAGCB("8", MF_CHECK | MF_GROUP3, lineStyleCallback, NULL),
     MI_SEPARATOR(),
-    MI_SIMPLECB("dashstyle", changeDashStyleAction, NULL),
+#define LW_GENERAL 5
+    MI_FLAGCB("linestyle", MF_CHECK | MF_GROUP3, lineStyleCallback, NULL),
 };
 
 static PaintMenuItem sizeMenu[] =
@@ -159,12 +170,13 @@ typedef struct {
     int click, mask, added, nbrushes;
     void *iconlist;
     GC gc;
-    Widget paint;
+    Widget form, form1, form2;
+    Widget bar, paint, box;
     Widget r_arrow, g_arrow, view1, view2, active, icon, fat;
     Widget cpick, edit;
     Widget curCheck;
     Widget sizeChecks[XtNumber(sizeMenu) - 1];
-    Widget vport, patternbox, patternlist;
+    Widget vport, vport2, patternbox, patternlist;
     Pixel marked;
     Pixmap iconpix;
     Palette *map;
@@ -318,17 +330,18 @@ setPatternColorsIcon(LocalInfo * l)
 static void
 initViews(LocalInfo * info)
 {
-    Display *dpy = NULL;
+    Display *dpy;
     Pixel p;
-    Pixmap pix, npix, opix, olpix;
-    int rule, orule, olrule;
+    Pixmap pix, npix, opix = None, olpix = None;
+    int rule=-1, orule=-1, olrule=-1;
+
     info->mask = 0;
     info->active = info->view1;
     info->edit = None;
 
     XtVaGetValues(info->fat, XtNfillRule, &orule, XtNpattern, &opix, 
 		  XtNlineFillRule, &olrule, XtNlinePattern, &olpix, NULL);
-
+ 
     if (info->paint) {
         dpy = XtDisplay(info->paint);
         XtVaGetValues(info->paint, XtNfillRule, &rule, NULL);
@@ -367,13 +380,14 @@ initViews(LocalInfo * info)
         XtVaSetValues(info->view1, XtNbackgroundPixmap, XtUnspecifiedPixmap,
                                    XtNbackground, p, NULL);
         XtVaSetValues(info->view2, XtNbackgroundPixmap, XtUnspecifiedPixmap,
-                                   XtNbackground, p, NULL);
+		                   XtNbackground, p, NULL);
         XtVaSetValues(info->fat, XtNfillRule, FillSolid, 
 		                 XtNforeground, p, 
                                  XtNlineFillRule, FillSolid, 
 		                 XtNlineForeground, p, NULL);
     }
 
+    dpy = XtDisplay(info->fat);
     if (orule==FillTiled && opix) XFreePixmap(dpy, opix);
     if (olrule==FillTiled && olpix) XFreePixmap(dpy, olpix);
 
@@ -382,7 +396,7 @@ initViews(LocalInfo * info)
     if (rule == FillSolid && p != BlackPixelOfScreen(XtScreen(info->view1)))
         ColorPickerSetPixel(info->cpick, p);
     setPatternColorsIcon(info);
-    setCanvasColorsIcon(info->paint);
+    setCanvasColorsIcon(info->paint, NULL);
 }
 
 /*
@@ -414,7 +428,8 @@ saveConfigOkCallback(Widget il, LocalInfo * info, char *file)
     }
     fprintf(fd, "reset\n\n");
 
-    GetDefaultWH(&w, &h);
+    w = Global.default_width;
+    h = Global.default_height;
     fprintf(fd, "DefaultSize %dx%d\n\n", w, h);
 
     for (i = 0; i < nchildren; i++) {
@@ -485,6 +500,7 @@ loadConfigOkCallback(Widget il, LocalInfo * info, char *file)
     info->patternlist = None;
     info->patternbox = XtVaCreateManagedWidget("patternRack",
 			            boxWidgetClass, info->vport,
+				    XtNforceBars, True,
 			            NULL);
     makePaletteArea(info, rcInfo);
     FreeRC(rcInfo);
@@ -524,7 +540,7 @@ activePixel(LocalInfo * l, Pixel p)
 	ColorPickerSetPixel(l->cpick, p);
     PaletteSetInvalid(l->map, p);
     setPatternColorsIcon(l);
-    setCanvasColorsIcon(l->paint);
+    setCanvasColorsIcon(l->paint, NULL);
 }
 
 static void
@@ -532,8 +548,8 @@ patternUpdate(PatternInfo * info)
 {
     char *w_fr, *w_fg, *w_pat;
     Display *dpy;
-    Pixmap pix;
-    int rule;
+    Pixmap pix=None, npix;
+    int rule=-1;
 
     if (!info->li->active) return;
     if (info->li->active == info->li->view1) {
@@ -558,13 +574,18 @@ patternUpdate(PatternInfo * info)
                           w_pat, dupPixmap(dpy, info->pixmap), NULL);
 	    if (rule==FillTiled && pix) XFreePixmap(dpy, pix);
 	}
+        dpy = XtDisplay(info->li->fat);
+        XtVaGetValues(info->li->fat, 
+                      w_fr, &rule, w_pat, &pix, NULL);
+        npix =  dupPixmap(dpy,info->pixmap);
         XtVaSetValues(info->li->fat, 
-                      w_fr, FillTiled, w_pat, info->pixmap, NULL);
+                      w_fr, FillTiled, w_pat, npix, NULL);
         XtVaSetValues(info->li->active, 
-                      XtNbackgroundPixmap, info->pixmap, NULL);
+                      XtNbackgroundPixmap, npix, NULL);
+        if (rule==FillTiled && pix) XFreePixmap(dpy, pix);
     }
     setPatternColorsIcon(info->li);
-    setCanvasColorsIcon(info->li->paint);
+    setCanvasColorsIcon(info->li->paint, NULL);
 }
 
 static void
@@ -637,7 +658,7 @@ unmarkAll(LocalInfo *l)
     int width, height;
 
     if (l->edit) {
-#ifdef XAW95
+#if defined(XAW3D) || defined(XAW95)
         XtVaGetValues(l->patternbox, XtNborder, &p, NULL);
 #else
         p = BlackPixelOfScreen(XtScreen(l->patternbox));
@@ -669,6 +690,58 @@ cac(LocalInfo * info, int inW, int inH)
     }
 }
 
+void
+patternResized(Widget w, LocalInfo * l, XConfigureEvent * event, Boolean * flg)
+{
+    Dimension width, height;
+    Dimension height1, height2;
+
+    XFlush(XtDisplay(l->form));   
+    w = XtParent(l->form);
+    if (!w) return;
+    XtVaGetValues(w, XtNwidth, &width, XtNheight, &height, NULL);
+    if (width<400) width = 400;
+    if (height<300) height = 300;
+#ifdef XAW3D
+    height1 = ((height-61)*265)/480;
+#ifdef XAW3DG
+    if (height1<271) height1 = 271;
+#else
+    if (height1<268) height1 = 268;   
+#endif   
+    height2 = ((height-61)*208)/480;
+#else   
+    height1 = ((height-59)*276)/480;
+    if (height1<276) height1 = 276;    
+    height2 = ((height-59)*196)/480;
+#endif   
+    XtResizeWidget(l->form, width-8, height1, 1);
+    XtMoveWidget(l->form, 4, 54);
+    XtResizeWidget(l->form1, width-8, height2, 1);
+    XtMoveWidget(l->form1, 4, 60 + height1);      
+    XtResizeWidget(l->form2, width-14, height2-8, 0);
+    XtMoveWidget(l->form2, 4, 4);   
+#ifdef XAWPLAIN
+    XtResizeWidget(l->vport, width-345, height1-32, 0);   
+    XtResizeWidget(l->box, width-345, height1-32, 0);
+#endif
+#ifdef XAW3D   
+    XtResizeWidget(l->vport, width-330, height1-18, 1);
+    XtResizeWidget(l->box, width-330, height1-18, 0);
+#endif
+#ifdef XAW95
+    XtResizeWidget(l->vport, width-345, height1-32, 0);
+    XtResizeWidget(l->box, width-348, height1-35, 0);   
+#endif
+    XtResizeWidget(l->patternbox, width-16, height2-8, 0);   
+    XtVaSetValues(l->patternbox, XtNwidth, width-16, XtNheight, height2-8, 
+		  NULL);
+    XtResizeWidget(l->vport2, width-16, height2-8, 0);   
+    XtVaSetValues(l->vport2, XtNwidth, width-16, XtNheight, height2-8, 
+		  NULL);
+    XtMoveWidget(l->vport2, 0, 0);
+}
+
 static void
 loadPattern(PatternInfo * info)
 {
@@ -687,6 +760,7 @@ loadPattern(PatternInfo * info)
         PwPutPixmap(l->fat, info->pixmap);
         cac(l, width, height);
     }
+    patternResized(info->li->fat, info->li, NULL, 0);   
 }
 
 void
@@ -698,6 +772,7 @@ markPattern(LocalInfo *l)
 		           XtNwidth, width-BSP, 
                            XtNheight, height-BSP, 
 		           XtNborderWidth, BSP/2+BWD, NULL);
+    patternResized(l->fat, l, NULL, 0);
 }
 
 static void
@@ -723,6 +798,7 @@ deletePatternCB(Widget mb, Widget w)
     PatternInfo *info, *ainfo;
     Widget parent;
     WidgetList children;
+    Pixmap pix;
     int nchild;
 
     XtVaGetValues(w, XtNradioData, &info, NULL);
@@ -737,7 +813,9 @@ deletePatternCB(Widget mb, Widget w)
     /*	
      *	Make sure there are enough widgets
      */
-    XtVaGetValues(parent = XtParent(info->widget),
+    parent = XtParent(info->widget);
+    if (!parent) return;
+    XtVaGetValues(parent,
 		  XtNnumChildren, &nchild,
 		  XtNchildren, &children,
 		  NULL);
@@ -746,7 +824,17 @@ deletePatternCB(Widget mb, Widget w)
 	return;
     }
     if (w == info->li->edit) info->li->edit = 0;
+    XtVaGetValues(info->li->fat, XtNpattern, &pix, NULL);
+    if (pix == info->pixmap) {
+       XtVaSetValues(info->li->fat, XtNfillRule, FillSolid, NULL);
+    }
+    XtVaGetValues(info->li->fat, XtNlinePattern, &pix, NULL);
+    if (pix == info->pixmap) {
+       XtVaSetValues(info->li->fat, XtNlineFillRule, FillSolid, NULL);
+    }   
+    XtUnmanageChild(info->widget);   
     XtDestroyWidget(info->widget);
+    patternResized(w, info->li, NULL, 0);
 }
 
 static void
@@ -775,6 +863,7 @@ markPatternCB(Widget mb, Widget w)
                      XtNwidth, width-BSP,
                      XtNheight, height-BSP,
                      XtNborderWidth, BSP/2+BWD, NULL);
+    patternResized(w, info->li, NULL, 0);
 }
 
 static void
@@ -813,6 +902,7 @@ unmarkPatternCB(Widget mb, Widget w)
         unmarkAll(info->li);
     if (w == info->widget)
         XawToggleUnsetCurrent(info->widget);
+    patternResized(w, info->li, NULL, 0);   
 }
 
 static void
@@ -838,7 +928,7 @@ AddPattern(LocalInfo *info, Pixmap pix, Pixel pxl)
     Widget icon;
     Display *dpy = XtDisplay(info->fat);
     PatternInfo *pi = XtNew(PatternInfo);
-    int i, depth;
+    int i, depth, nw, nh, x, y;
     char *nm;
     char *str;
     XrmValue val;
@@ -891,13 +981,15 @@ AddPattern(LocalInfo *info, Pixmap pix, Pixel pxl)
 	XSetForeground(dpy, info->gc, pi->pixel);
 	XFillRectangle(dpy, pi->shownPixmap, 
                        info->gc, 0, 0, pattern_min_wh, pattern_min_wh);
+        nw = nh = pattern_min_wh;
 #endif
     } else {
         nm = "pattern";
+        nw = pi->width ;
+        nh = pi->height;
         if (pi->width < pattern_min_wh || pi->height < pattern_min_wh) {
-	int nw = (pi->width < pattern_min_wh) ? pattern_min_wh : pi->width;
-	int nh = (pi->height < pattern_min_wh) ? pattern_min_wh : pi->height;
-	int x, y;
+	   nw = (pi->width < pattern_min_wh) ? pattern_min_wh : pi->width;
+	   nh = (pi->height < pattern_min_wh) ? pattern_min_wh : pi->height;
 
 	pi->shownPixmap = 
             XCreatePixmap(dpy, DefaultRootWindow(dpy), nw, nh, depth);
@@ -928,6 +1020,8 @@ AddPattern(LocalInfo *info, Pixmap pix, Pixel pxl)
 				        XtNradioGroup, info->patternlist,
 					XtNtranslations, trans,
 					XtNradioData, pi,
+				        XtNwidth, nw+10+COLSP,
+				        XtNheight, nh+10,
 					NULL);
 #ifdef HAVE_COLTOG
     }
@@ -944,13 +1038,15 @@ AddPattern(LocalInfo *info, Pixmap pix, Pixel pxl)
 
     pi->widget = icon;
     XtAddCallback(icon, XtNdestroyCallback, (XtCallbackProc) freePatternInfo,
-		  (XtPointer) pi);
+		  (XtPointer) pi);   
     return icon;
 }
 
 static void 
 cpickCallback(Widget w, LocalInfo * l, Pixel p)
 {
+    if (Global.popped_up) return;
+
     if (l->patternlist && !l->click && !l->edit)
         XawToggleUnsetCurrent(l->patternlist);
     l->click = 0;
@@ -981,6 +1077,34 @@ changeSolid(PatternInfo * info, Pixel p)
 #endif
 }
 
+static void 
+setAsBackgroundCallback(Widget w, LocalInfo * l, XPointer * junk2)
+{
+    Pixel p;
+    XColor *color;
+    XColor xc;
+    int r, g, b;
+
+    if (Global.popped_up) return;
+
+    if (l->map->isMapped) {
+        color = ColorPickerGetXColor(l->cpick);
+        p = PaletteAlloc(l->map, color);
+    } else
+        XtVaGetValues(l->active, XtNbackground, &p, NULL);
+
+    xc.pixel = p;
+    xc.flags = DoRed | DoGreen | DoBlue;
+    XQueryColor(XtDisplay(w), l->map->cmap, &xc);
+    r = (xc.red >> 8) & 0xff;
+    g = (xc.green >> 8) & 0xff;
+    b = (xc.blue >> 8) & 0xff;
+
+    Global.background = (r<<16) | (g<<8) | b;
+    XtVaSetValues(l->paint, XtNbackground, p, NULL);
+    activePixel(l, p);
+}
+
 static void 
 colorSelectCallback(Widget w, LocalInfo * l, XPointer * junk2)
 {
@@ -993,6 +1117,8 @@ colorSelectCallback(Widget w, LocalInfo * l, XPointer * junk2)
     int r=0, g=0, b=0;
     int i, nchildren;
 
+    if (Global.popped_up) return;
+
     if (l->map->isMapped) {
         color = ColorPickerGetXColor(l->cpick);
         p = PaletteAlloc(l->map, color);
@@ -1049,6 +1175,35 @@ colorSelectCallback(Widget w, LocalInfo * l, XPointer * junk2)
     XtVaGetValues(icon, XtNradioData, &pi, NULL);
     XawToggleSetCurrent(l->patternlist, (XtPointer) pi);
     if (l->paint) AddItemToCanvasPalette(l->paint, p, None);
+    patternResized(l->fat, l, NULL, 0);
+}
+
+static void 
+popupHandler(Widget w, LocalInfo * l, XEvent * event, Boolean * flg)
+{
+    KeySym keysym;
+    int len;
+    char buf[21];
+
+    if (Global.popped_up) {
+        if (event->type == ButtonRelease)
+            PopdownMenusGlobal();
+        event->type = None;
+        return;
+    }
+
+    if (event->type == KeyPress) {
+        len = XLookupString((XKeyEvent *)event, 
+                            buf, sizeof(buf) - 1, &keysym, NULL);
+        switch (keysym) {
+            case XK_Escape:
+                PopdownMenusGlobal();
+                event->type = None;
+                break;
+            default:
+                break;
+	}
+    }
 }
 
 static void 
@@ -1057,6 +1212,13 @@ switchCallback(Widget w, LocalInfo * l, XEvent * event, Boolean * flg)
     Pixel p;
     int side = 1;
 
+    if (Global.popped_up) {
+        if (event->type == ButtonRelease)
+            PopdownMenusGlobal();
+        event->type = None;
+        return;
+    }
+
     if (event->type != ButtonRelease);
     if (w == l->icon) {
         if (l->paint)
@@ -1081,7 +1243,7 @@ switchCallback(Widget w, LocalInfo * l, XEvent * event, Boolean * flg)
             activePixel(l, p);
     }
     setPatternColorsIcon(l);
-    setCanvasColorsIcon(l->paint);
+    setCanvasColorsIcon(l->paint, NULL);
 }
 
 static void 
@@ -1101,7 +1263,7 @@ grabPatternCallback(Widget w, XtPointer infoArg, XtPointer junk2)
 
     XtVaGetValues(info->fat, XtNcolormap, &cmap, NULL);
     pix = None;
-    if (ImageToPixmapCmap(image, info->fat, &pix, cmap))
+    if (ImageToPixmapCmap(image, info->fat, &pix, cmap, 0))
 	PwPutPixmap(info->fat, pix);
 }
 
@@ -1125,7 +1287,7 @@ checkPatternLink(Widget w, int mode)
             XtVaSetValues(info->fat, XtNlineWidth, lw, NULL);
             XtVaSetValues(info->fat, XtNbackground, p, NULL);
             setPatternColorsIcon(info);
-	    setCanvasColorsIcon(info->paint);
+	    setCanvasColorsIcon(info->paint, NULL);
             MenuCheckItem(lineMenu[(lw/2<5)? (lw/2) : LW_GENERAL].widget, True);
 	}
     }
@@ -1146,6 +1308,7 @@ static void
 closeShellCallback(Widget w, XtPointer infoArg, XtPointer junk2)
 {
     LocalInfo *info = (LocalInfo *) infoArg;
+    PopdownMenusGlobal();
     if (info->iconpix)
         XFreePixmap(XtDisplay(info->icon), info->iconpix);
     info->iconpix = None;
@@ -1186,6 +1349,7 @@ selectPatternCallback(Widget w, XtPointer infoArg, XtPointer junk2)
 	if (pi) patternUpdate(pi);
     }
     if (info->paint) AddItemToCanvasPalette(info->paint, None, pix);
+    patternResized(info->fat, info, NULL, 0);      
 }
 
 static void 
@@ -1199,7 +1363,9 @@ asbrushPatternCallback(Widget w, XtPointer infoArg, XtPointer junk2)
     PwRegionFinish(info->fat, True);
     PwGetPixmap(info->fat, &pix, NULL, NULL);
 
-    image = PixmapToImage(info->fat, pix, info->map->cmap);
+    image = PixmapToImage(info->fat, 
+                          dupPixmap(XtDisplay(Global.toplevel), pix),
+                          info->map->cmap);
 
     i = Global.nbrushes;
     Global.nbrushes = i+1;
@@ -1207,10 +1373,8 @@ asbrushPatternCallback(Widget w, XtPointer infoArg, XtPointer junk2)
        realloc(Global.brushes, Global.nbrushes * sizeof(void *));
     Global.brushes[i] = (void *) image;
 
-    if (Global.brushpopup) {
-        XtUnrealizeWidget(Global.brushpopup);
-	Global.brushpopup = None;
-    }
+    /* Refresh Brush Popup */
+    Global.brushpopup = createBrushDialog(Global.toplevel);
     BrushSelect(Global.toplevel);
 }
 
@@ -1223,7 +1387,7 @@ readFileCallback(Widget paint, XtPointer fileArg, XtPointer imageArg)
     int  width, height, depth;
 
     /*  XXX - allocating a new colormap! */
-    if (ImageToPixmap(image, paint, &pix, &cmap)) {
+    if (ImageToPixmap(image, paint, &pix, &cmap, 0)) {
         GetPixmapWHD(XtDisplay(paint), pix, &width, &height, &depth);
         XtVaSetValues(paint, XtNdrawWidth, width,
 		      XtNdrawHeight, height, NULL);
@@ -1236,7 +1400,7 @@ readFileCallback(Widget paint, XtPointer fileArg, XtPointer imageArg)
 static void 
 readCallback(Widget w, XtPointer paint, XtPointer junk)
 {
-    GetFileName((Widget) paint, False, NULL, readFileCallback, NULL);
+    GetFileName((Widget) paint, 0, NULL, readFileCallback, NULL);
 }
 
 static void 
@@ -1246,9 +1410,9 @@ sizeOkChoiceCallback(Widget w, LocalInfo * l, TextPromptInfo * info)
     int height = atoi(info->prompts[1].rstr);
 
     if (width <= 0 || height <= 0) {
-	Notice(w, "Invalid width or height,\nmust be greater than 0");
+	Notice(w, msgText[INVALID_WIDTH_OR_HEIGHT_MUST_BE_GREATER_THAN_ZERO]);
     } else if (width > 128 || height > 128) {
-	Notice(w, "Invalid width or height,\nmust be less than 129");
+	Notice(w, msgText[INVALID_WIDTH_OR_HEIGHT_MUST_BE_LESS_THAN_HUNDRED_TWENTY_NINE]);
     } else {
 	XtVaSetValues(l->fat, XtNdrawWidth, width,
 		      XtNdrawHeight, height,
@@ -1303,7 +1467,7 @@ sizeCallback(Widget w, XtPointer larg, XtPointer junk)
     height = -1;
     sscanf(lbl, "%dx%d", &width, &height);
     if (width <= 0 || height <= 0 || width >= 256 || height >= 256) {
-	Notice(w, "Invalid width/height specification must be between 0..256");
+	Notice(w, msgText[INVALID_WIDTH_OR_HEIGHT_MUST_BE_BETWEEN_ZERO_AND_TWO_HUNDRED_FIFTY_SIX]);
 	return;
     }
     MenuCheckItem(l->curCheck = w, True);
@@ -1375,6 +1539,7 @@ lookupPatternCallback(Widget w, XtPointer infoArg, XtPointer junk2)
 	    p = PaletteAlloc(info->map, &col);
     }
     list = info->patternlist;
+    if (!list || !XtParent(list)) return;
 
     XtVaGetValues(XtParent(list),
 		  XtNnumChildren, &nchildren,
@@ -1450,7 +1615,8 @@ makePaletteArea(LocalInfo * info, RCInfo * rcInfo)
 
 	pix = None;
 	rcInfo->images[i]->refCount++;
-	ImageToPixmapCmap(rcInfo->images[i], info->fat, &pix, info->map->cmap);
+	ImageToPixmapCmap(rcInfo->images[i], info->fat, &pix,
+                          info->map->cmap, 0);
 	AddPattern(info, pix, 0);
     }
 }
@@ -1479,9 +1645,9 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
     static Pixmap r_arrow_pix = 0, g_arrow_pix = 0;
     LocalInfo *info = NULL; 
     Display * dpy;
-    Widget topf, box, vport, form, form2, prform;
+    Widget topf;
     Widget fat, norm, grabPattern, selectPattern, asbrushPattern;
-    Widget bar, cpick, cancel;
+    Widget cpick, cancel;
     Colormap cmap;
     Pixel white = WhitePixelOfScreen(XtScreen(w));
     Pixel black = BlackPixelOfScreen(XtScreen(w));
@@ -1521,6 +1687,7 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
     StateSetBusyWatch(True);
     info = (LocalInfo *) XtMalloc(sizeof(LocalInfo));
     Global.patterninfo = (void *) info;
+   
     info->paint = w;
     info->gc = XCreateGC(dpy, XtWindow(w), 0, 0);
     info->iconlist = None;
@@ -1556,15 +1723,15 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
     topf = XtVaCreateManagedWidget("form", formWidgetClass, 
                                    Global.patternshell, NULL);
 
-    bar = MenuBarCreate(topf, XtNumber(menuBar), menuBar);
-    XtVaSetValues(bar, XtNvertDistance, 9, NULL);
+    info->bar = MenuBarCreate(topf, XtNumber(menuBar), menuBar);
+    XtVaSetValues(info->bar, XtNvertDistance, 9, NULL);
 
     info->r_arrow = XtVaCreateManagedWidget("r_arrow", coreWidgetClass, topf,
                                       XtNwidth, r_arrow_width,
                                       XtNheight, r_arrow_height,
 				      XtNvertDistance, 15,
                                       XtNhorizDistance, 12,
-                                      XtNfromHoriz, bar,
+                                      XtNfromHoriz, info->bar,
                                       XtNborderWidth, 0,
                                       NULL);
 
@@ -1606,17 +1773,17 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
 				      XtNhorizDistance, 15,
                                       NULL);
 
-    form = XtVaCreateManagedWidget("patternbox", formWidgetClass, topf,
+    info->form = XtVaCreateManagedWidget("patternbox", formWidgetClass, topf,
 				   XtNfromVert, info->view1,
 				   NULL);
 
     cpick = None;
 
-    if (!map->isMapped || !map->readonly || map->ncolors > 256)
-	cpick = ColorPickerPalette(form, map, &white);
+    if (!map->isMapped || !map->readonly_ || map->ncolors > 256)
+	cpick = ColorPickerPalette(info->form, map, &white);
 
     grabPattern = XtVaCreateManagedWidget("grab",
-					 commandWidgetClass, form,
+					 commandWidgetClass, info->form,
 					 XtNhorizDistance, 25,
 					 XtNvertDistance, 4,
 				         XtNleft, XtChainLeft,
@@ -1625,7 +1792,7 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
 					 NULL);
 
     selectPattern = XtVaCreateManagedWidget("select",
-				       commandWidgetClass, form,
+				       commandWidgetClass, info->form,
 				       XtNvertDistance, 4,
 				       XtNfromHoriz, grabPattern,
 				       XtNright, XtChainRight,
@@ -1634,7 +1801,7 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
 				       NULL);
 
     asbrushPattern = XtVaCreateManagedWidget("asbrush",
-				       commandWidgetClass, form,
+				       commandWidgetClass, info->form,
 				       XtNvertDistance, 4,
 				       XtNfromHoriz, selectPattern,
 				       XtNright, XtChainRight,
@@ -1642,7 +1809,8 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
 				       XtNbottom, XtChainBottom,
 				       NULL);
 
-    vport = XtVaCreateManagedWidget("viewport", viewportWidgetClass, form,
+    info->vport = XtVaCreateManagedWidget("viewport", viewportWidgetClass, 
+				 info->form,
 				 XtNfromVert, grabPattern,
 				 XtNvertDistance, 4,
 				 XtNallowVert, True,
@@ -1653,37 +1821,43 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
 				 XtNright, XtChainRight,
 			         XtNtop, XtChainTop,
 				 XtNbottom, XtChainBottom,
-				 XtNwidth, 282, XtNheight, 244,
+				 XtNwidth, 286,
+				 XtNheight, 290,
 				 NULL);
 
     if (cpick != None) {
-        XtVaSetValues(vport, XtNfromHoriz, cpick, NULL);
+        XtVaSetValues(info->vport, XtNfromHoriz, cpick, NULL);
         XtVaSetValues(grabPattern, XtNfromHoriz, cpick, NULL);
     }
 
-    box = XtVaCreateManagedWidget("paintBox", boxWidgetClass, vport,
-			    XtNbackgroundPixmap, GetBackgroundPixmap(vport),
+    info->box = XtVaCreateManagedWidget("paintBox", boxWidgetClass, info->vport,
+			    XtNbackgroundPixmap, 
+				  GetBackgroundPixmap(info->vport),
                             XtNtop, XtChainTop,
                             XtNbottom, XtChainBottom,
 			    XtNorientation, XtorientVertical,
 			    NULL);
 
     XtVaGetValues(w, XtNlineWidth, &lw, NULL);
-    fat = XtVaCreateManagedWidget("paint", paintWidgetClass, box,
+   
+    fat = XtVaCreateManagedWidget("paint", paintWidgetClass, info->box,
 			    XtNbackground, white,
 			    XtNdrawWidth, 24,
 			    XtNdrawHeight, 24,
 			    XtNlineWidth, lw,
 			    XtNfillRule, FillSolid,
+			    XtNlineFillRule, FillSolid,
 			    NULL);
-    norm = XtVaCreateManagedWidget("norm", paintWidgetClass, box,
+    norm = XtVaCreateManagedWidget("norm", paintWidgetClass, info->box,
 			    XtNbackground, white,
                             XtNfromVert, fat,
 			    XtNhorizDistance, 0,
 			    XtNpaint, fat,
 			    XtNfillRule, FillSolid,
+			    XtNlineFillRule, FillSolid,
 			    XtNzoom, 1,
 			    NULL);
+   
     OperationSetPaint(fat);
     ccpAddStdPopup(fat, NULL);
 
@@ -1697,10 +1871,24 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
 	}
 	XtAddCallback(ColorPickerGetSelectWidget(cpick), XtNcallback,
 	    (XtCallbackProc) colorSelectCallback, (XtPointer) info);
+	XtAddCallback(ColorPickerGetAsBgWidget(cpick), XtNcallback,
+	    (XtCallbackProc) setAsBackgroundCallback, (XtPointer) info);
     }
 
-    if (cpick!=None)
+    if (cpick != None) {
         XtVaSetValues(grabPattern, XtNfromHoriz, cpick, NULL);
+        XtInsertRawEventHandler(cpick, 
+           ButtonPressMask | ButtonReleaseMask, False,
+	   (XtEventHandler) popupHandler, (XtPointer) info,
+           XtListHead);
+    }
+
+    XtAddEventHandler(topf, 
+        KeyPressMask | ButtonPressMask | ButtonReleaseMask, False,
+        (XtEventHandler) popupHandler, (XtPointer) info);
+    XtAddEventHandler(info->form, 
+        KeyPressMask | ButtonPressMask | ButtonReleaseMask, False,
+        (XtEventHandler) popupHandler, (XtPointer) info);
 
     XtAddEventHandler(info->view1, ButtonPressMask | ButtonReleaseMask, False,
         (XtEventHandler) switchCallback, (XtPointer) info);
@@ -1723,21 +1911,21 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
     XtAddCallback(paletteMenu[DELETE_PATTERN].widget, XtNcallback,
                   (XtCallbackProc) deletePatternCallback, (XtPointer) info);
 
-    XtAddCallback(XtNameToWidget(bar, "canvas.canvasMenu.close"),
+    XtAddCallback(XtNameToWidget(info->bar, "canvas.canvasMenu.close"),
 		  XtNcallback, closeShellCallback, (XtPointer) info);
-    XtAddCallback(XtNameToWidget(bar, "canvas.canvasMenu.save"),
+    XtAddCallback(XtNameToWidget(info->bar, "canvas.canvasMenu.save"),
 		  XtNcallback, StdSaveFile, (XtPointer) fat);
-    XtAddCallback(XtNameToWidget(bar, "canvas.canvasMenu.read"),
+    XtAddCallback(XtNameToWidget(info->bar, "canvas.canvasMenu.read"),
 		  XtNcallback, readCallback, (XtPointer) fat);
 
-    ccpAddUndo(XtNameToWidget(bar, "edit.editMenu.undo"), fat);
-    ccpAddCut(XtNameToWidget(bar, "edit.editMenu.cut"), fat);
-    ccpAddCopy(XtNameToWidget(bar, "edit.editMenu.copy"), fat);
-    ccpAddPaste(XtNameToWidget(bar, "edit.editMenu.paste"), fat);
-    ccpAddClear(XtNameToWidget(bar, "edit.editMenu.clear"), fat);
-    ccpAddDuplicate(XtNameToWidget(bar, "edit.editMenu.dup"), fat);
+    ccpAddUndo(XtNameToWidget(info->bar, "edit.editMenu.undo"), fat);
+    ccpAddCut(XtNameToWidget(info->bar, "edit.editMenu.cut"), fat);
+    ccpAddCopy(XtNameToWidget(info->bar, "edit.editMenu.copy"), fat);
+    ccpAddPaste(XtNameToWidget(info->bar, "edit.editMenu.paste"), fat);
+    ccpAddClear(XtNameToWidget(info->bar, "edit.editMenu.clear"), fat);
+    ccpAddDuplicate(XtNameToWidget(info->bar, "edit.editMenu.dup"), fat);
 
-    XtAddCallback(XtNameToWidget(bar, "edit.editMenu.all"),
+    XtAddCallback(XtNameToWidget(info->bar, "edit.editMenu.all"),
 		  XtNcallback, StdSelectAllCallback, (XtPointer) fat);
 
     XtAddCallback(sizeMenu[SZ_N1].widget,
@@ -1778,31 +1966,33 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
     XtAddCallback(grabPattern, XtNcallback,
 		  (XtCallbackProc) grabPatternCallback, (XtPointer) info);
 
+   
     GraphicAdd(fat);
     GraphicAdd(norm);
     XtVaSetValues(fat, XtNmenuwidgets, NULL, NULL);
     XtVaSetValues(norm, XtNmenuwidgets, NULL, NULL);
 
-    form2 = XtVaCreateManagedWidget("form2",
+    info->form1 = XtVaCreateManagedWidget("form1",
 				   formWidgetClass, topf,
-				   XtNfromVert, form,
+				   XtNfromVert, info->form,
 				   NULL);
 
-    prform = XtVaCreateManagedWidget("patternRackForm",
-				   formWidgetClass, form2,
+    info->form2 = XtVaCreateManagedWidget("patternRackForm",
+				   formWidgetClass, info->form1,
 				   XtNborderWidth, 0,
 				   XtNleft, XtChainLeft,
 				   XtNright, XtChainRight,
 				   NULL);
 
-    info->vport = XtVaCreateManagedWidget("viewport2",
-				    viewportWidgetClass, prform,
+    info->vport2 = XtVaCreateManagedWidget("viewport2",
+				    viewportWidgetClass, info->form2,
 				    XtNallowVert, True,
 				    XtNuseBottom, True,
 				    XtNuseRight, True,
 				    NULL);
     info->patternbox = XtVaCreateManagedWidget("patternRack",
-			            boxWidgetClass, info->vport,
+			            boxWidgetClass, info->vport2,
+				    XtNforceBars, False,
 			            NULL);
 
     /*
@@ -1812,15 +2002,22 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
     copyPaletteArea(info, pixels, patterns, npixels, npatterns);
     XtPopup(Global.patternshell, XtGrabNone);
    
-    XtUnmanageChild(info->cpick);
+    XtUnmanageChild(info->bar);
+    if (info->cpick)
+       XtUnmanageChild(info->cpick);
     XtUnmanageChild(grabPattern);
     XtUnmanageChild(selectPattern);
     XtUnmanageChild(asbrushPattern);
     XtUnmanageChild(info->r_arrow);
     XtUnmanageChild(info->g_arrow);
+    XtUnmanageChild(info->form);   
+    XtUnmanageChild(info->form1);
+    XtUnmanageChild(info->form2);
+    XtUnmanageChild(info->vport2);
     XtUnmanageChild(info->view1);
     XtUnmanageChild(info->view2);
     XtUnmanageChild(info->icon);
+    XtUnmanageChild(cancel);
 
     XtVaGetValues(Global.patternshell, XtNbackground, &bgpix, NULL);
     if (!r_arrow_pix) {
@@ -1841,13 +2038,20 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
     info->iconpix = XCreatePixmap(dpy, DefaultRootWindow(dpy), 
                                    ICONWIDTH, ICONHEIGHT, depth);
 
-    XMapWindow(dpy, XtWindow(info->cpick));
+    XMapWindow(dpy, XtWindow(info->bar));
+    if (info->cpick)
+       XMapWindow(dpy, XtWindow(info->cpick));
     XMapWindow(dpy, XtWindow(grabPattern));
     XMapWindow(dpy, XtWindow(selectPattern));
     XMapWindow(dpy, XtWindow(asbrushPattern));
+    XMapWindow(dpy, XtWindow(info->form));   
+    XMapWindow(dpy, XtWindow(info->form1));
+    XMapWindow(dpy, XtWindow(info->form2));      
     XMapWindow(dpy, XtWindow(info->view1));
     XMapWindow(dpy, XtWindow(info->view2));
+    XMapWindow(dpy, XtWindow(info->vport2));
     XMapWindow(dpy, XtWindow(info->icon));
+    XMapWindow(dpy, XtWindow(cancel));   
     XMapWindow(dpy, XtWindow(info->r_arrow));
     XUnmapWindow(dpy, XtWindow(info->g_arrow));
     initViews(info);
@@ -1855,6 +2059,10 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
     XtVaGetValues(info->paint, XtNlineWidth, &nargs, NULL);
     MenuCheckItem(lineMenu[(nargs/2<5)? (nargs/2) : LW_GENERAL].widget, True);
     StateSetBusyWatch(False);
+    XtAddEventHandler(Global.patternshell, StructureNotifyMask, False,
+		      (XtEventHandler) patternResized, (XtPointer) info);
+
+    XtSetMinSizeHints(Global.patternshell, 210, 186);
 }
 
 
