@@ -118,7 +118,7 @@ static PaintMenuItem formatMenu[] =
     MI_FLAG("B5", MF_CHECK | MF_GROUP1),
 };
 
-static int paper_sizes[XtNumber(formatMenu)][2] = {
+int paper_sizes[XtNumber(formatMenu)][2] = {
       {/* Letter */           612,  792},
       {/* Legal */            612, 1008},
       {/* Statement */        396,  612},
@@ -203,13 +203,6 @@ typedef struct {
     Widget formatChecks[XtNumber(externMenu)];
 } ExternInfo;
 
-typedef struct {
-    float wpercent, hpercent;
-    int wbbox, hbbox, width, height, rx, ry, wsubdiv, hsubdiv;
-    Boolean orient, eps, gray, compress, pdf;
-    char *title;
-} PageInfo;
-
 static PrintInfo *printinfo = NULL;
 static ExternInfo *externinfo = NULL;
 static char *printcmd_str = NULL;
