@@ -74,18 +74,33 @@ static int tempIndex = -1;
 static char *tempName[20];
 static RCInfo *baseInfo = NULL;
 
+void
+openTempDir(char *buf)
+{
+    char *tmp, *home;
+
+    if ((tmp = getenv("TMPDIR")) != NULL) {
+        sprintf(buf, "mkdir -p \"%s\"", tmp);
+        system(buf);
+        strncpy(buf, tmp, 200);
+    } else
+    if ((home = getenv("HOME")) != NULL) {
+        sprintf(buf, "%s/.xpaint", home);
+        mkdir(buf, 0777);
+        sprintf(buf, "%s/.xpaint/tmp", home);
+        mkdir(buf, 0777);
+    } else
+        strncpy(buf, P_tmpdir, 200);
+}
+
 FILE *
-openTemp(char **np)
+openTempFile(char **np)
 {
     int fd;
-    char *tmp;
     char buf[256];
 
-    if ((tmp = getenv("TMPDIR")) == NULL)
-        tmp = P_tmpdir;
-
-    strncpy(buf, tmp, 220);
-    buf[220] = '\0';
+    openTempDir(buf);
+    buf[200] = '\0';
     strcat(buf, "/XPaint-XXXXXX");
     fd = mkstemp(buf);
     if (fd == -1) return NULL;
@@ -97,7 +112,7 @@ openTemp(char **np)
 }
 
 void 
-removeTemp(void)
+removeTempFile(void)
 {
     if (tempIndex < 0)
 	return;
@@ -252,7 +267,7 @@ readRC(RCInfo ** info, char *file)
 	    if (EQ(argv[1], "BeginData")) {
 		FILE *ofd;
 
-		ofd = openTemp(&nm);
+		ofd = openTempFile(&nm);
 		while (fgets(buf, sizeof(buf), fd) != NULL) {
 		    if (strncmp(buf, "EndData", 7) == 0)
 			break;
@@ -262,7 +277,7 @@ readRC(RCInfo ** info, char *file)
 		if (ofd != NULL) {
 		    fclose(ofd);
 		} else {
-		    removeTemp();
+		    removeTempFile();
 		    continue;
 		}
 	    } else {
@@ -272,7 +287,7 @@ readRC(RCInfo ** info, char *file)
 	    if ((image = ReadMagic(nm)) != NULL)
 		addImage(*info, image, isBrush);
 
-	    removeTemp();
+	    removeTempFile();
 	}
     }
 
@@ -367,7 +382,7 @@ ReadDefaultRC()
 	/*
 	**  Load the default RC
 	 */
-	if (!defaultDone && ((fd = openTemp(&tn)) != NULL)) {
+	if (!defaultDone && ((fd = openTempFile(&tn)) != NULL)) {
 	    for (i = 0; i < XtNumber(defaultRC); i++) {
 		fputs(defaultRC[i], fd);
 		putc('\n', fd);
@@ -375,7 +390,7 @@ ReadDefaultRC()
 	    fclose(fd);
 
 	    readRC(&info, tn);
-	    removeTemp();
+	    removeTempFile();
 	}
 	/*
 	**  Load ~/.XPaintrc
