@@ -13,6 +13,9 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <X11/Xlib.h>
+#include <X11/Intrinsic.h>
+#include "xpaint.h"
 #include "image.h"
 #include "rwTable.h"
 
@@ -35,6 +38,10 @@
 
 #define LM_to_uint(a,b)			(((b)<<8)|(a))
 
+extern int file_isSpecialImage;
+extern int file_transparent;
+extern int file_numpages;
+
 struct {
     unsigned int Width;
     unsigned int Height;
@@ -43,26 +50,26 @@ struct {
     unsigned int ColorResolution;
     unsigned int Background;
     unsigned int AspectRatio;
-    int GrayScale;
+    int GrayColorScheme;
 } GifScreen;
 
 static struct {
     int transparent;
+    int index;
     int delayTime;
     int inputFlag;
     int disposal;
 } Gif89 = {
-    -1, -1, -1, 0
+    0, -1, -1, -1, 0
 };
 
-
 static int ReadColorMap(FILE * fd, int number,
 			unsigned char buffer[3][MAXCOLORMAPSIZE], int *flag);
 static int DoExtension(FILE * fd, int label);
 static int GetDataBlock(FILE * fd, unsigned char *buf);
 static int GetCode(FILE * fd, int code_size, int flag);
 static int LWZReadByte(FILE * fd, int flag, int input_code_size);
-static Image *ReadImage(FILE * fd, int len, int height, int,
+static Image *ReadImage(FILE * fd, int width, int height, int cmapSize,
 			unsigned char cmap[3][MAXCOLORMAPSIZE],
 			int gray, int interlace, int ignore);
 
@@ -92,28 +99,30 @@ ReadGIF(char *file)
 {
     unsigned char buf[16];
     unsigned char c;
+    unsigned char *ptr;
+    unsigned char transp_index = 0;
     unsigned char localColorMap[3][MAXCOLORMAPSIZE];
+    int i, j, k, transparent = 0, numpage;
     int grayScale;
     int useGlobalColormap;
     int bitPixel;
     int imageCount = 0;
     char version[4];
-    int imageNumber = 1;
     FILE *fd = fopen(file, "r");
-    Image *image = NULL;
+    Image *image, *output = NULL;
 
     if (fd == NULL)
-	return image;
+	return NULL;
 
     if (!ReadOK(fd, buf, 6)) {
 	RWSetMsg("error reading magic number");
 	fclose(fd);
-	return image;
+	return NULL;
     }
     if (strncmp((char *) buf, "GIF", 3) != 0) {
 	RWSetMsg("not a GIF file");
 	fclose(fd);
-	return image;
+	return NULL;
     }
     strncpy(version, (char *) buf + 3, 3);
     version[3] = '\0';
@@ -121,12 +130,12 @@ ReadGIF(char *file)
     if ((strcmp(version, "87a") != 0) && (strcmp(version, "89a") != 0)) {
 	RWSetMsg("bad version number, not '87a' or '89a'");
 	fclose(fd);
-	return image;
+	return NULL;
     }
     if (!ReadOK(fd, buf, 7)) {
 	RWSetMsg("failed to read screen descriptor");
 	fclose(fd);
-	return image;
+	return NULL;
     }
     GifScreen.Width = LM_to_uint(buf[0], buf[1]);
     GifScreen.Height = LM_to_uint(buf[2], buf[3]);
@@ -137,32 +146,39 @@ ReadGIF(char *file)
 
     if (BitSet(buf[4], LOCALCOLORMAP)) {	/* Global Colormap */
 	if (ReadColorMap(fd, GifScreen.BitPixel, GifScreen.ColorMap,
-			 &GifScreen.GrayScale)) {
+			 &GifScreen.GrayColorScheme)) {
 	    RWSetMsg("error reading global colormap");
 	    fclose(fd);
-	    return image;
+	    return NULL;
 	}
     }
-    do {
+
+    numpage = Global.numpage;
+
+    while (1) {
 	if (!ReadOK(fd, &c, 1)) {
-	    RWSetMsg("EOF / read error on image data");
+	    if (!output) RWSetMsg("EOF / read error on image data");
 	    fclose(fd);
-	    return image;
+	    goto the_end;
 	}
 	if (c == ';') {		/* GIF terminator */
-	    if (imageCount < imageNumber) {
+	    if (imageCount < numpage) {
 		RWSetMsg("only %d image%s found in file",
 			 imageCount, imageCount > 1 ? "s" : "");
 		fclose(fd);
-		return image;
+		return NULL;
 	    }
+            goto the_end;
 	}
 	if (c == '!') {		/* Extension */
 	    if (!ReadOK(fd, &c, 1)) {
-		RWSetMsg("EOF / read error on extention function code");
-		return image;
+		RWSetMsg("EOF / read error on extension function code");
+		goto the_end;
 	    }
 	    DoExtension(fd, c);
+            transparent = Gif89.transparent;
+            if (transparent)
+                transp_index = Gif89.index;
 	    continue;
 	}
 	if (c != ',') {		/* Not a valid start character */
@@ -173,7 +189,7 @@ ReadGIF(char *file)
 	if (!ReadOK(fd, buf, 9)) {
 	    RWSetMsg("couldn't read left/top/width/height");
 	    fclose(fd);
-	    return image;
+	    goto the_end;
 	}
 	useGlobalColormap = !BitSet(buf[8], LOCALCOLORMAP);
 
@@ -185,21 +201,31 @@ ReadGIF(char *file)
 		return NULL;
 	    }
 	    image = ReadImage(fd, LM_to_uint(buf[4], buf[5]),
-			      LM_to_uint(buf[6], buf[7]),
+		 	      LM_to_uint(buf[6], buf[7]),
 			      bitPixel, localColorMap, grayScale,
 			      BitSet(buf[8], INTERLACE),
-			      imageCount != imageNumber);
+			      imageCount != numpage);
 	} else {
 	    image = ReadImage(fd, LM_to_uint(buf[4], buf[5]),
 			      LM_to_uint(buf[6], buf[7]),
 			      GifScreen.BitPixel, GifScreen.ColorMap,
-			      GifScreen.GrayScale, BitSet(buf[8], INTERLACE),
-			      imageCount != imageNumber);
+			      GifScreen.GrayColorScheme, 
+                              BitSet(buf[8], INTERLACE),
+			      imageCount != numpage);
 	}
+        if (image) output = image;
+        file_numpages = imageCount;
+        file_isSpecialImage = (file_numpages>1) || transparent;
     }
-    while (image == NULL);
 
-    return image;
+ the_end :
+    if (transparent && output) {
+        file_transparent = 1;
+        ImageMakeMask(output); 
+	for (i=0; i<output->width*output->height; i++)
+	    output->alpha[i] = ((output->data[i]==transp_index)? 0:255);
+    }
+    return output;
 }
 
 static int
@@ -258,8 +284,10 @@ DoExtension(FILE *fd, int label)
 	Gif89.disposal = (buf[0] >> 2) & 0x7;
 	Gif89.inputFlag = (buf[0] >> 1) & 0x1;
 	Gif89.delayTime = LM_to_uint(buf[1], buf[2]);
-	if ((buf[0] & 0x1) != 0)
-	    Gif89.transparent = buf[3];
+	if ((buf[0] & 0x1) != 0) {
+	    Gif89.transparent = 1;
+	    Gif89.index = buf[3];
+	}
 
 	while (GetDataBlock(fd, (unsigned char *) buf) != 0);
 	return FALSE;
@@ -442,7 +470,7 @@ LWZReadByte(FILE *fd, int flag, int input_code_size)
 }
 
 static Image *
-ReadImage(FILE * fd, int len, int height, int cmapSize,
+ReadImage(FILE * fd, int width, int height, int cmapSize,
 	  unsigned char cmap[3][MAXCOLORMAPSIZE],
 	  int gray, int interlace, int ignore)
 {
@@ -469,17 +497,17 @@ ReadImage(FILE * fd, int len, int height, int cmapSize,
 	while (LWZReadByte(fd, FALSE, c) >= 0);
 	return NULL;
     }
-    image = ImageNewCmap(len, height, cmapSize);
+    image = ImageNewCmap(width, height, cmapSize);
 
     for (i = 0; i < cmapSize; i++)
 	ImageSetCmap(image, i, cmap[CM_RED][i],
 		     cmap[CM_GREEN][i], cmap[CM_BLUE][i]);
 
     while ((v = LWZReadByte(fd, FALSE, c)) >= 0) {
-	image->data[xpos + ypos * len] = v;
+	image->data[xpos + ypos * width] = v;
 
 	++xpos;
-	if (xpos == len) {
+	if (xpos == width) {
 	    xpos = 0;
 	    if (interlace) {
 		switch (pass) {
@@ -508,7 +536,7 @@ ReadImage(FILE * fd, int len, int height, int cmapSize,
 			ypos = 1;
 			break;
 		    default:
-			goto fini;
+			goto finished;
 		    }
 		}
 	    } else {
@@ -519,7 +547,7 @@ ReadImage(FILE * fd, int len, int height, int cmapSize,
 	    break;
     }
 
-  fini:
+  finished:
 
     return image;
 }
