@@ -22,6 +22,9 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+extern int file_isSpecialImage;
+extern int file_transparent;
+
 extern char * GetShareDir();
 extern void *dl_filter;
 extern void *dl_proc;
@@ -96,7 +99,12 @@ ReadScriptC(char *file)
        if (proc) {
           fprintf(stderr, "Executing C-script ImageCreate() ... \n");
           image = proc();
-          if (!image)
+          if (image) {
+             if (image->alpha) {
+	        file_isSpecialImage = 1;
+	        file_transparent = 1;
+	     }
+	  } else
              fprintf(stderr, "C-script procedure created invalid image !!\n");
        } else {
           image = NULL;
