@@ -60,7 +60,7 @@ TestPNG(char *file)  /* gets called a LOT on the first image:  brushes? */
     fread(header, 1, 8, fp);
     fclose(fp);
 
-    return png_check_sig( (unsigned char*) header, 8);
+    return !png_sig_cmp( (unsigned char*) header, 0, 8);
 }
 
 
