@@ -15,9 +15,11 @@
 /* $Id: readWritePS.c,v 1.14 2005/03/20 20:15:34 demailly Exp $ */
 
 /* default ghostcript resolution
- * replace by 72 if 180 is too much to wish for ...
+ * replace by 72 if 300 is too much to wish for ...
  */
 
+#define GS_RESOLUTION 300.0
+
 #define MIN(a,b)       (((a) < (b)) ? (a) : (b))
 
 #include <stdio.h>
@@ -59,14 +61,18 @@ typedef unsigned char byte;
 typedef unsigned int  uint;
 
 extern int file_isSpecialImage;
+extern int file_transparent;
 extern int file_numpages;
 extern int file_force;
 extern int file_bbox;
+extern int paper_sizes[15][2];
 
 extern void * xmalloc(size_t n);
 extern FILE * openTempFile(char **np);
 extern void removeTempFile(void);
+extern void openTempDir(char *);
 extern Image * ReadPNG(char *file);
+extern int TestPNG(char *file);
 extern void setPageSize(int *w, int *h);
 extern double GetDpi();
 
@@ -665,16 +671,16 @@ ReadPS_(char *file)
 {
     char *tmp;
     char scan[80];
-    char rad[260];
-    static char last_rad[260] = "";
+    char rad[512];
+    static char * last_rad = NULL;
     static int  last_type = 0;
     char buffer[2048];
+    char *sc, *dc;
     FILE *fp;
     int wth, hth, type_doc;
     int i, j;
-    double x1, y1, x2, y2, res;
-    char *dp;
-    Image *image;
+    double x1=0.0, y1=0.0, x2=1000.0, y2=1414.0, res;
+    Image *image = NULL;
 
     fp = fopen(file, "rb");
     if (!fp) return NULL;
@@ -690,10 +696,19 @@ ReadPS_(char *file)
         return NULL; /* should not happen anyway ... */
     
     file_isSpecialImage = 1;
+    file_transparent = 1;
 
-    strncpy(rad, file, 256),
-    rad[256] = '\0';
-    if ((dp=strrchr(rad, '.'))) *dp = '\0';
+    openTempDir(rad);
+    strcat(rad, "/");
+    strcpy(buffer, file);
+    if ((sc=strrchr(buffer, '/'))) {
+        *sc = '\0';
+        ++sc;
+    } else
+        sc = buffer;
+    if ((dc=strrchr(sc, '.')))
+        *dc = '\0';
+    strcat(rad, sc);
    
     if (type_doc>=PDF_FMT && type_doc<=DVI_FMT) { /* TeX et al documents */
        if (type_doc==PDF_FMT)
@@ -714,15 +729,18 @@ ReadPS_(char *file)
            sprintf(buffer, 
                "dvips %s -o %s.ps ", file, rad);
        strcat(rad, ".ps");
-       if (file_force || type_doc != last_type || strcmp(last_rad, rad)) {
+       if (file_force || type_doc != last_type || 
+           !last_rad || strcmp(last_rad, rad)) {
+	   printf("Running:\n%s\n", buffer);
            system(buffer);
            file_force = 0;
-       } else {
-	   strcpy(last_rad, rad);
+           last_rad = (char *)realloc(last_rad, strlen(rad)+2);
+           strcpy(last_rad, rad);
            last_type = type_doc;
        }
     } else
        strcpy(rad, file);
+
     wth = Global.default_width;
     hth = Global.default_height;
 
@@ -732,8 +750,9 @@ ReadPS_(char *file)
        sprintf(buffer, 
           "inkscape %s --export-height=%d --export-png=%s",
 	  file, hth, tmp);
+       printf("Running:\n%s\n", buffer);
        system(buffer);
-       image = ReadPNG(tmp);
+       if (TestPNG(tmp)) image = ReadPNG(tmp);
        removeTempFile();
        return image;
     }
@@ -747,32 +766,61 @@ ReadPS_(char *file)
        fgets(buffer, 2040, fp);
        ++j;
        if (!strncasecmp(buffer, "%%BoundingBox:", 14)) {
-	     sscanf(buffer, "%s %lf %lf %lf %lf", scan, &x1, &y1, &x2, &y2);
-          i = 1;
+	   sscanf(buffer, "%s %lf %lf %lf %lf", scan, &x1, &y1, &x2, &y2);
+           i = 1;
        }
        if (!strncasecmp(buffer, "%%HiResBoundingBox:", 19)) {
-          sscanf(buffer, "%s %lf %lf %lf %lf", scan, &x1, &y1, &x2, &y2);
-          i = 2;
+           sscanf(buffer, "%s %lf %lf %lf %lf", scan, &x1, &y1, &x2, &y2);
+           i = 2;
        }
        if (!strncmp(buffer, "%%Pages:",8) && !strstr(buffer,"(atend)")) {
-	     file_numpages = atoi(buffer+8);
+	   file_numpages = atoi(buffer+8);
        }
        if (file_numpages>=0 && i==2) break;
        if (file_numpages>=0 && i==1 && j>=20) break;
     }
     if (fp) fclose(fp);
 
-    if (!i || !file_bbox) {        
+    if (!i || file_bbox==0) {
        x1 = y1 = 0;
        setPageSize(&i, &j);
        x2 = (double)i;
        y2 = (double)j;
     }
-    if ((res=GetDpi()) == 0)
-          res = 300.0 * MIN(wth / (x2-x1), hth / (y2-y1));
+    if (i && file_bbox==1) {
+       int mx = 1191, my = 1684;
+       x1 = y1 = 0;
+       if (x2<y2) {
+          for (j = 0; j<15; j++) {
+	      if (x2<paper_sizes[j][0]+1 && y2<paper_sizes[j][1]+1 &&
+		  paper_sizes[j][0]<=mx && paper_sizes[j][1]<=my) {
+		 mx = paper_sizes[j][0];
+		 my = paper_sizes[j][1];
+	      } 
+	  }
+          x2 = (double) mx;
+          y2 = (double) my;
+       }
+       if (x2>=y2) {
+          for (j = 0; j<15; j++) {
+	      if (y2<paper_sizes[j][0]+1 && x2<paper_sizes[j][1]+1 &&
+		  paper_sizes[j][0]<=mx && paper_sizes[j][1]<=my) {
+		 mx = paper_sizes[j][0];
+		 my = paper_sizes[j][1];
+	      } 
+	  }
+          y2 = (double) mx;
+          x2 = (double) my;
+       }
+    }
+    if (i && file_bbox==2)
+       x1 = y1 = 0;
 
-    wth = (int) (0.5+res*(x2-x1)/72.0);
-    hth = (int) (0.5+res*(y2-y1)/72.0);
+    if ((res=GetDpi()) == 0)
+          res = GS_RESOLUTION * MIN(wth/x2, hth/y2);
+
+    wth = (int) (0.5+res*x2/72.0);
+    hth = (int) (0.5+res*y2/72.0);
 
     /*
     if (type_doc==PDF_FMT)
@@ -784,15 +832,65 @@ ReadPS_(char *file)
                 tmp, rad);
     */
     sprintf(buffer, 
-                "psselect -p %d %s | "
+                "psselect -p%d %s | "
                 "gs -sDEVICE=pngalpha -r%f -g%dx%d -dNOPAUSE -dQUIET "
                 "-sOutputFile=%s - -c quit", 
                 Global.numpage, rad,
 		res, wth, hth, tmp);
+    printf("Running:\n%s\n", buffer);
     system(buffer);
 
-    image = ReadPNG(tmp);
+    if (TestPNG(tmp)) image = ReadPNG(tmp);
     removeTempFile();
+
+    if (!image) return NULL;
+
+    if (image->alpha) {
+        i = image->width*image->height;
+        j = 1;
+        while (i>0)
+	    if (image->alpha[--i] != 0xff) { j = 0; break; }
+        if (j) {
+            free(image->alpha);
+            image->alpha = NULL;
+	}
+    }
+    if (!image->alpha)
+        file_transparent = 0;
+
+    i = (int) (res*x1/72.0);
+    j = (int) (res*y1/72.0);
+
+    /* crop left and bottom bordures in case of BBox= */
+    if (file_bbox==3 && (i>=0 && j>=0) && (i+j>0)) {
+        int dw, dh, wth1, hth1;
+        unsigned char *data;
+        dw = i;
+        dh = j;
+        wth1 = wth - i;
+        hth1 = hth - j;
+        data = (unsigned char*) malloc(wth1*hth1*image->scale);
+        if (!data) return image;
+        for (j=0; j<hth1; j++)
+	    memcpy(data+j*wth1*image->scale, 
+                   image->data+(dw+j*wth)*image->scale, wth1*image->scale);
+        free(image->data);
+        image->data = data;
+        if (image->alpha) {
+            data = (unsigned char*) malloc(wth1*hth1);
+            if (!data) {
+	        free(image->alpha);
+                image->alpha = NULL;
+	    }
+            for (j=0; j<hth1; j++)
+	        memcpy(data+j*wth1, image->alpha+dw+j*wth, wth1);
+            free(image->alpha);
+            image->alpha = data;
+	}
+        image->width = wth1;
+        image->height = hth1;
+    }
+
     return image;
 }
 
