@@ -17,6 +17,8 @@
 #if defined(HAVE_PARAM_H)
 #include <sys/param.h>
 #endif
+#include <sys/types.h>
+#include <sys/stat.h>
 
 #include <stdio.h>
 #include "image.h"
