@@ -12,7 +12,7 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: writeGIF.c,v 1.11 2000/10/02 20:53:00 torsten Exp $ */
+/* $Id: writeGIF.c,v 1.21 2005/03/20 20:15:34 demailly Exp $ */
 
 /* ppmtogif.c - read a portable pixmap and produce a GIF file
 **
@@ -38,11 +38,11 @@
 #endif
 
 #include <stdio.h>
+#include <errno.h>
+
 #include "image.h"
 #include "rwTable.h"
-#ifdef __EMX__
-#include <errno.h>
-#endif
+
 
 #define MAXCOLORS 256
 
@@ -84,9 +84,6 @@ static Image *image;
 int
 WriteGIF(char *file, Image * outImage)
 {
-#ifndef __EMX__
-    extern int errno;
-#endif
     int r[MAXCOLORS], g[MAXCOLORS], b[MAXCOLORS];
     int i, bpp;
     FILE *fp;
