@@ -11,9 +11,11 @@
 #include <X11/StringDefs.h>
 
 #include "XPAINT_SHAREDIR/include/Paint.h"
+#include "XPAINT_SHAREDIR/include/PaintP.h"
 #include "XPAINT_SHAREDIR/include/xpaint.h"
 #include "XPAINT_SHAREDIR/include/graphic.h"
 #include "XPAINT_SHAREDIR/include/image.h"
 #include "XPAINT_SHAREDIR/include/misc.h"
 #include "XPAINT_SHAREDIR/include/region.h"
+#include "XPAINT_SHAREDIR/include/text.h"
 
