@@ -21,22 +21,20 @@
 #include <X11/Intrinsic.h>
 #include "Paint.h"
 #include "image.h"
+#include "graphic.h"
 #include "messages.h"
+#include "misc.h"
+#include "region.h"
 
 /* extern procedures */
 extern Image * outputImage;
-extern void ClipboardSetImage(Widget w, Image * image);
-extern void StdPasteCallback(Widget w, XtPointer paintArg, XtPointer junk);
-extern Widget makeGraphicShell(Widget wid);
-extern Widget graphicCreate(Widget shell, int width, int height, int zoom,
-       Pixmap pix, Colormap cmap);
+extern char *routine;
 
 /*  Leave arguments as globals, since there are so many of them.
  *  They'll only be referenced in process_args and main.
  */
 static Colormap private_cmap = 0;
 
-
 /*
  *  createEventWindow returns the ID of a InputOnly window that covers
  *  the given window.
@@ -136,14 +134,14 @@ find_window(Display *dpy, int flag, int x, int y,
 	stopW = childW;
     }
 
+    if (!flag)
+      findW = initW;
+
     XGetWindowAttributes(dpy, findW, &wa);
     *width = wa.width;
     *height = wa.height;
     private_cmap = wa.colormap;
 
-    if (!flag)
-      findW = initW;
-
     XTranslateCoordinates(dpy, findW, DefaultRootWindow(dpy),
 			  0, 0, u, v, &stopW);
 }
@@ -186,7 +184,7 @@ void get_region(Display *dpy, Window win,
 		     ButtonPressMask,
 		     GrabModeAsync, GrabModeAsync, None, up_left_curs, 
 		     CurrentTime) != 0) {
-        fprintf(stderr, msgText[CANNOT_GRAB_POINTER]);
+        fprintf(stderr, "%s", msgText[CANNOT_GRAB_POINTER]);
 	return;
     }
 
@@ -306,13 +304,13 @@ void get_region(Display *dpy, Window win,
     if (done == 1) {
       int u;
       if (*width==0 && *height==0)
-	  find_window(dpy, 0, init_x, init_y, x, y, width, height);
+	  find_window(dpy, 0, init_x, init_y, x, y, (int*) width, (int*) height);
       else
 	  find_window(dpy, 0, init_x, init_y, &u, &u, &u, &u);
     }
 
     if (done == 2)
-        find_window(dpy, 1, init_x,init_y, x, y, width, height);
+        find_window(dpy, 1, init_x,init_y, x, y, (int*) width, (int*) height);
 }
 
 /* 
@@ -343,12 +341,13 @@ Pixmap get_pixmap_region(Display *dpy, int screen, Window win, GC gc,
     *width = reg_width;
     *height = reg_height;
 
-    if (*width==0 || *height==0) return 0;
+    if (*width==0 || *height==0) return None;
 
     /* Use the depth of `win' for the depth of the pixmap */
 	  
     XGetGeometry (dpy, win, &junk_root, &junk_left, &junk_top,
-		  &junk_width, &junk_height, &junk_border_width, depth);
+		  (unsigned int*) &junk_width, (unsigned int*) &junk_height,
+		  (unsigned int*) &junk_border_width, depth);
 
     pixmap_returned = XCreatePixmap(dpy, 
 				    DefaultRootWindow(dpy),
@@ -362,32 +361,6 @@ Pixmap get_pixmap_region(Display *dpy, int screen, Window win, GC gc,
     return pixmap_returned;
 }
 
-
-/* 
- *  X error handler for,  mainly for future use
- */
-
-int privateErrorHandler(Display *dpy, XErrorEvent *myerr)
-{
-    char msg[82];  
-
-    /* if we get a BadAlloc error, it's probably 'cuz we couldn't allocate
-     * the pixmap.  Tell the user this.
-     */
-    if (myerr->error_code == None) return 0;
-    fprintf(stderr,"snapshot:\n"); 
-    if (myerr->error_code == BadAlloc) {
-        fprintf(stderr, msgText[INSUFFICIENT_RESOURCES_FOR_OPERATION]);
-	fprintf(stderr, msgText[PROBABLY_NOT_ENOUGH_MEMORY_TO_ALLOCATE_PIXMAP]);
-    }
-
-    /* Now print out the error string that X generates */
-    
-    XGetErrorText(dpy, myerr->error_code, msg, 80);
-    fprintf(stderr, "%s %s\n", msgText[ERROR_CODE], msg);
-    return 0;
-}
-
 void
 SnapshotImage(Widget w, XtPointer paintArg, int flag)
 {
@@ -396,15 +369,17 @@ SnapshotImage(Widget w, XtPointer paintArg, int flag)
     GC copy_gc;
     unsigned long copy_gc_valuemask;
     XGCValues copy_gc_values;
+    Widget ww;
     Window window_to_snap;            
     Pixmap  snap_pixmap;
     int reg_x, reg_y;
     unsigned int reg_width, reg_height, reg_depth;
     Image *image;
 
+    routine = "snapshot";
     dpy = XtDisplayOfObject(w);
     screen = DefaultScreen(dpy);
-    XSetErrorHandler(privateErrorHandler);
+    XSetErrorHandler(privateXErrorHandler);
 
     /* start with root window for now */
     window_to_snap = XRootWindow(dpy, screen);
@@ -431,21 +406,25 @@ SnapshotImage(Widget w, XtPointer paintArg, int flag)
     if (!snap_pixmap || reg_width==0 || reg_height==0)
         return;
 
-    image = PixmapToImage(w, snap_pixmap, private_cmap);
+    if (paintArg) ww = (Widget)paintArg; else ww = w;
+    image = PixmapToImage(ww, snap_pixmap, private_cmap);
+    if (snap_pixmap) XFreePixmap(dpy, snap_pixmap);
+   
     if (!image) return;
 
     if (flag) {
-        ClipboardSetImage(w, image);
-	StdPasteCallback(w, paintArg, (XtPointer) NULL);
-        XtVaSetValues((Widget)paintArg, XtNdirty, True, NULL);
+        ClipboardSetImage(ww, image);
+	StdPasteCallback(ww, paintArg, (XtPointer) NULL);
+        XtVaSetValues(ww, XtNdirty, True, NULL);
     }
     else {
         Widget paint;
-        XFreePixmap(dpy, snap_pixmap);
-        ImageToPixmap(image, w, &snap_pixmap, &private_cmap);
+        snap_pixmap = None;
+        ImageToPixmap(image, w, &snap_pixmap, &private_cmap, 0);
 	paint = (Widget) graphicCreate(makeGraphicShell(w), 
-		      reg_width, reg_height, -1, snap_pixmap, private_cmap);
-        XFreePixmap(dpy, snap_pixmap);
+		      reg_width, reg_height, -1, 
+				       snap_pixmap, private_cmap, NULL);
+        if (snap_pixmap) XFreePixmap(dpy, snap_pixmap);
         XtVaSetValues(paint, XtNdirty, True, NULL);
     }
 }
