@@ -64,15 +64,16 @@ XDrawContour(Widget w, Drawable d, LocalInfo *l, int flag)
     Display *dpy;
     GC gc;
     XPoint *xp;
-    int n, i, j, imax, i0, i1=0, zoom, rx, ry;
+    int n, i, j, imax, i0, i1=0, zoom, rx, ry, bool;
     int u, v, up, vp;
     FLOAT ax=0.0, ay=0.0, bx, by, cx, cy, r=0.0, s, t;
 
     dpy = XtDisplay(w);
+    bool = (d==XtWindow(w));
 
     if (flag == FINISH && d == l->pixmap) {
         zoom = l->zoom;
-	rx = l->rx;
+        rx = l->rx;
 	ry = l->ry;
     } else {
         zoom = 1;
@@ -98,12 +99,26 @@ XDrawContour(Widget w, Drawable d, LocalInfo *l, int flag)
 
     if (n == 1) {
          xp = (XPoint *) xmalloc(2 * sizeof(XPoint));
-	 xp[0].x = rx + l->points[0].x / zoom;
-	 xp[0].y = ry + l->points[0].y / zoom;
-	 xp[1].x = rx + l->points[1].x / zoom;
-	 xp[1].y = ry + l->points[1].y / zoom;
-         XDrawLine(dpy, d, gc, xp[0].x, xp[0].y, xp[0].x, xp[0].y);
-         XDrawLine(dpy, d, gc, xp[0].x, xp[0].y, xp[1].x, xp[1].y);
+         if (zoom>0) {
+	     xp[0].x = rx + l->points[0].x / zoom;
+	     xp[0].y = ry + l->points[0].y / zoom;
+	     xp[1].x = rx + l->points[1].x / zoom;
+	     xp[1].y = ry + l->points[1].y / zoom;
+	 } else {
+	     xp[0].x = rx + l->points[0].x;
+	     xp[0].y = ry + l->points[0].y;
+	     xp[1].x = rx + l->points[1].x;
+	     xp[1].y = ry + l->points[1].y;
+	 }
+         if (bool && l->zoom<0) {
+	     XDrawLine(dpy, d, gc, xp[0].x/(-l->zoom), xp[0].y/(-l->zoom), 
+                                   xp[0].x/(-l->zoom), xp[0].y/(-l->zoom));
+             XDrawLine(dpy, d, gc, xp[0].x/(-l->zoom), xp[0].y/(-l->zoom),
+                                   xp[1].x/(-l->zoom), xp[1].y/(-l->zoom));
+	 } else {
+             XDrawLine(dpy, d, gc, xp[0].x, xp[0].y, xp[0].x, xp[0].y);
+             XDrawLine(dpy, d, gc, xp[0].x, xp[0].y, xp[1].x, xp[1].y);
+	 }
          if (flag == FINISH && d == l->pixmap)
              for (i=0; i<=1; i++) UndoGrow(w, xp[i].x, xp[i].y);
          l->mode = ModeCtrl;
@@ -198,8 +213,13 @@ XDrawContour(Widget w, Drawable d, LocalInfo *l, int flag)
     if (flag == FINISH) {
         n = 0;
         xp = (XPoint *) xmalloc((imax * SUBDIV + 2) * sizeof(XPoint));
-	xp[0].x = rx + l->points[0].x / zoom;
-	xp[0].y = ry + l->points[0].y / zoom;
+        if (zoom>0) {
+	    xp[0].x = rx + l->points[0].x / zoom;
+	    xp[0].y = ry + l->points[0].y / zoom;
+	} else {
+	    xp[0].x = rx + l->points[0].x;
+	    xp[0].y = ry + l->points[0].y;
+	}
 	if (d == l->pixmap)
             UndoGrow(w, xp[0].x, xp[0].y);
         for (i = 0; i < imax; i++) {
@@ -210,8 +230,13 @@ XDrawContour(Widget w, Drawable d, LocalInfo *l, int flag)
 			     t*t*(t*l->points[i+1].x+3.0*s*l->interm2[i].x));
 		v = (int) (s*s*(s*l->points[i].y+3.0*t*l->interm1[i].y)+
 		       t*t*(t*l->points[i+1].y+3.0*s*l->interm2[i].y));
-		xp[++n].x = rx + u / zoom;
-		xp[n].y = ry + v / zoom;
+                if (zoom>0) {
+		    xp[++n].x = rx + u / zoom;
+		    xp[n].y = ry + v / zoom;
+		} else {
+		    xp[++n].x = rx + u;
+		    xp[n].y = ry + v;
+		}
 		if (d == l->pixmap)
 		    UndoGrow(w, xp[n].x, xp[n].y);
 	    }
@@ -241,8 +266,15 @@ XDrawContour(Widget w, Drawable d, LocalInfo *l, int flag)
 			     t*t*(t*l->points[i+1].x+3.0*s*l->interm2[i].x));
 		v = (int) (s*s*(s*l->points[i].y+3.0*t*l->interm1[i].y)+
 		       t*t*(t*l->points[i+1].y+3.0*s*l->interm2[i].y));
-		XDrawLine(dpy, d, gc, up, vp, up, vp);
-		XDrawLine(dpy, d, gc, up, vp, u, v);
+                if (bool && l->zoom<0) {
+		    XDrawLine(dpy, d, gc, up/(-l->zoom), vp/(-l->zoom), 
+                                          up/(-l->zoom), vp/(-l->zoom));
+		    XDrawLine(dpy, d, gc, up/(-l->zoom), vp/(-l->zoom),
+                                          u/(-l->zoom), v/(-l->zoom));
+		} else {
+		    XDrawLine(dpy, d, gc, up, vp, up, vp);
+		    XDrawLine(dpy, d, gc, up, vp, u, v);
+		}
 	    }
 	}
     }
@@ -264,7 +296,9 @@ finish(Widget w, LocalInfo * l)
     --l->npoints;
 
     if (l->npoints > 0) {
-        SetCapAndJoin(w, l->fgc, JoinRound);
+        SetCapAndJoin(w, l->fgc,
+           ((Global.cap)?Global.cap-1:CapButt),
+           ((Global.join)?Global.join-1:JoinRound));
         if (!l->isFat)
            XDrawContour(w, XtWindow(w), l, FINISH);
         XDrawContour(w, l->pixmap, l, FINISH);
@@ -310,20 +344,30 @@ pressSplineBand(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 	  l->npoints = -2;
           return;
       }
+      XtVaGetValues(w, XtNzoom, &l->zoom, NULL);
       if (info->surface == opPixmap) {
 	l->isFat = info->isFat;
 	l->fgc = info->first_gc;
 	l->sgc = info->second_gc;
-	XtVaGetValues(w, XtNzoom, &l->zoom, NULL);
-        l->rx = info->x - l->points[0].x / l->zoom;
-        l->ry = info->y - l->points[0].y / l->zoom;
+        if (l->zoom>0) {
+            l->rx = info->x - l->points[0].x / l->zoom;
+            l->ry = info->y - l->points[0].y / l->zoom;
+	} else {
+	    l->rx = info->x - l->points[0].x;
+	    l->ry = info->y - l->points[0].y;
+	}
 	l->npoints = 0;
         l->mode = ModeCtrl;
         UndoStart(w, info);
         l->pixmap = info->drawable;
       } else {
-	l->points[0].x = event->x;
-	l->points[0].y = event->y;
+        if (l->zoom>0) {
+	    l->points[0].x = event->x;
+	    l->points[0].y = event->y;
+	} else {
+	    l->points[0].x = event->x * (-l->zoom);
+	    l->points[0].y = event->y * (-l->zoom);
+	}
         l->breakpt[0] = (ModeShift)? True : False;
       }
       return;        
@@ -376,12 +420,21 @@ releaseSplineBand(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 
     if ((event->button == Button1) && (info->surface == opWindow)) {
         check_modifiers(event);
-	if (l->npoints>0 &&
-            event->x==l->points[l->npoints-1].x &&
-	    event->y==l->points[l->npoints-1].y) return;
-        ++l->npoints;
-	l->points[l->npoints].x = event->x;
-	l->points[l->npoints].y = event->y;
+        if (l->zoom>0) {
+	    if (l->npoints>0 &&
+                event->x==l->points[l->npoints-1].x &&
+	        event->y==l->points[l->npoints-1].y) return;
+            ++l->npoints;
+	    l->points[l->npoints].x = event->x;
+	    l->points[l->npoints].y = event->y;
+	} else {
+	    if (l->npoints>0 &&
+                event->x==l->points[l->npoints-1].x*(-l->zoom) &&
+	        event->y==l->points[l->npoints-1].y*(-l->zoom)) return;
+            ++l->npoints;
+	    l->points[l->npoints].x = event->x * (-l->zoom);
+	    l->points[l->npoints].y = event->y * (-l->zoom);
+	}
     }
 }
 
@@ -401,15 +454,27 @@ motionSplineBand(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
        Global.escape = 0;       
        return;
     }
-   
-    if ((event->x != l->points[l->npoints].x) ||
-	(event->y != l->points[l->npoints].y)) {
-        check_modifiers((XButtonEvent *)event);
-        XDrawContour(w, info->drawable, l, ERASE);
-        l->points[l->npoints].x = event->x;
-        l->points[l->npoints].y = event->y;
-        l->breakpt[l->npoints-1] = (ModeShift)? True : False;
-	XDrawContour(w, info->drawable, l, DRAW);
+
+    if (l->zoom>0) {   
+        if ((event->x != l->points[l->npoints].x) ||
+	    (event->y != l->points[l->npoints].y)) {
+            check_modifiers((XButtonEvent *)event);
+            XDrawContour(w, info->drawable, l, ERASE);
+            l->points[l->npoints].x = event->x;
+            l->points[l->npoints].y = event->y;
+            l->breakpt[l->npoints-1] = (ModeShift)? True : False;
+  	    XDrawContour(w, info->drawable, l, DRAW);
+	}
+    } else {
+        if ((event->x != l->points[l->npoints].x*(-l->zoom)) ||
+	    (event->y != l->points[l->npoints].y*(-l->zoom))) {
+            check_modifiers((XButtonEvent *)event);
+            XDrawContour(w, info->drawable, l, ERASE);
+	    l->points[l->npoints].x = event->x * (-l->zoom);
+            l->points[l->npoints].y = event->y * (-l->zoom);
+            l->breakpt[l->npoints-1] = (ModeShift)? True : False;
+  	    XDrawContour(w, info->drawable, l, DRAW);
+	}
     }
 }
 
@@ -417,11 +482,11 @@ static
 LocalInfo * createLocalInfo()
 {
     LocalInfo * l;
-    l = (LocalInfo *) malloc(sizeof(LocalInfo));
-    l->points =  (XPoint *) malloc(MAXPOINTS*sizeof(XPoint));
-    l->interm1 = (XPoint *) malloc(MAXPOINTS*sizeof(XPoint));
-    l->interm2 = (XPoint *) malloc(MAXPOINTS*sizeof(XPoint));
-    l->breakpt = (Boolean *) malloc(MAXPOINTS*sizeof(Boolean));
+    l = (LocalInfo *) xmalloc(sizeof(LocalInfo));
+    l->points =  (XPoint *) xmalloc(MAXPOINTS*sizeof(XPoint));
+    l->interm1 = (XPoint *) xmalloc(MAXPOINTS*sizeof(XPoint));
+    l->interm2 = (XPoint *) xmalloc(MAXPOINTS*sizeof(XPoint));
+    l->breakpt = (Boolean *) xmalloc(MAXPOINTS*sizeof(Boolean));
     return l;
 }
 
