@@ -47,10 +47,16 @@
 
 extern char *routine;
 
+typedef struct {
+    unsigned int s1;
+    unsigned int s2;
+} CRCdata;
+
 typedef struct {
     int x, y;
-    Pixmap sourcepix, pix, mask;
+    Pixmap pix, mask;
     unsigned char *alpha;
+    CRCdata crcdata;
 } RegionData;
 
 extern struct Global_s {
@@ -68,6 +74,14 @@ extern struct Global_s {
     Boolean transparent;
     Time currentTime;
     XVisualInfo vis;
+    char * xft_name;   /* name of Xft font */
+    void * xft_font;
+    double xft_size;   /* size of font */
+    double xft_height; /* height of font */
+    double xft_ascent, xft_descent, xft_maxadv; 
+                       /* ascent, descent, max advance */
+    double xft_rot;    /* angle of rotation of font */
+    double xft_linespacing;  /* linespacing factor */
     unsigned int background;
     unsigned int alpha_mode, alpha_threshold, alpha_default;
     unsigned int default_width, default_height;
